'''
Created on 01.06.2010

@author: sschneider
'''
from IO import PDBReader, PDBWriter
from Structure import  Protein
from SurfPointCreator import CavityProbeIdentifier
import math, div

def deleteNarrowRegions(bfact, dist=1.2, nrNeig=4):
    dd = dist * dist
    for a in bfact.atoms:
        a.occ = 0.0
    for a in bfact.atoms:
        if a.occ >= nrNeig:
            continue
        for b in bfact.atoms:
            d = div.distanceNoSQRT(a, b) 
            if d < 0.1:
                continue
            if d <= dd:
                a.occ += 1
#                b.occ += 1
#                if a.occ >= nrNeig:
#                    break
    new = []
    for a in bfact.atoms:
        if a.occ >= nrNeig :
            new.append(a)
    return Protein.Protein(new)


def getNrNeighbours(at, possNeigs, dist=1.4, skip={}):
    dd = dist * dist
    count = 0
    for a in possNeigs:
        if a.at_nr == at.at_nr:
            continue
        if div.distanceNoSQRT(a, at) < dd:
            count += 1
    return count
        
def getNeighbours(at, possNeigs, dist=1.4, skip={}):
    dd = dist * dist
    neigs = []
    for a in possNeigs:
        if a.at_nr == at.at_nr:
            continue
        if div.distanceNoSQRT(a, at) < dd:
            neigs.append(a.copy())
    return neigs

def deleteContactAreas(bfact, dist=1.4, nrNeig=3):     
    lowNeigs = []
    nr = 1    
    for a in bfact.atoms:
        a.occ = 0.0
        a.at_nr = nr
        nr += 1
    for a in bfact.atoms:
        neigCount = getNrNeighbours(a, bfact.atoms, dist)        
        if neigCount < nrNeig:
            lowNeigs.append(a.copy())
    toDel = {}
    for spot in lowNeigs:
        neigs = getNeighbours(spot, bfact.atoms, dist)
        if len(neigs) == 0:
            toDel[spot.at_nr] = True
            continue
#        basicCount = float(len(neigs)) / base
        supCount = 0.0
        for n in neigs:
            nr = getNrNeighbours(n, bfact.atoms, dist)
            if nr >= nrNeig:
                supCount += 1.0
#            supCount += float(nr) / base
#        basicCount += supCount
#        basicCount /= (float(len(neigs)) + 1.0)
#        if basicCount < 0.5:
        if supCount / float(len(neigs)) < 0.5:
            toDel[spot.at_nr] = True
    ret = []
    for a in bfact.atoms:
        if a.at_nr not in toDel:
            a.at_nr = 1
            ret.append(a)
    return Protein.Protein(ret)
    
def reduceBordersFromSkinnestFirst(bfact):
    pass


def takeHighestFromBfact(bfact, perc=0.1):
    highest = {}
    nr = len(bfact.atoms) * perc
#    if nr > 20:
#        nr = 20
    while len(highest) < nr:
        high = -1110.0
        one = None
        for a in bfact.atoms:
            if a.at_nr in highest:
                continue
            if a.temp > high:
                high = a.temp
                one = a.copy()
        if one != None:
            highest[one.at_nr] = one 
        else:
            break
    return Protein.Protein([a for nr, a in highest.iteritems()])

def takeHighestFromBfactNoScale(bfact, perc=0.1):
    highest = {}
    nr = len(bfact.atoms) * perc
    
    while len(highest) < nr:
        high = -1110.0
        one = None
        for a in bfact.atoms:
            if a.at_nr in highest:
                continue
            if a.temp > high:
                high = a.temp
                one = a.copy()
        if one != None:
            highest[one.at_nr] = one 
        else:
            break
    return Protein.Protein([a for nr, a in highest.iteritems()])
"""
bfact; surface points with desolvation energy as temp factor
nrOf: minimum nr of elements in cluster to be accepted
distTo: defines the center to center distance between Surface points for which those
surface points are in one cluster. 
"""
def takeLowestFromBfact(bfact, nrOf, distTo, frac=05):
    lowest = {}
    for i in range(len(bfact.atoms)):
        bfact.atoms[i].at_nr = i + 1
    nr = int(len(bfact.atoms) * frac)
#    if nr < 10:
#        nr = 10
#    if nr > 300:
#        nr = 300
    if nr > len(bfact.atoms):
        nr = len(bfact.atoms)        
    while len(lowest) < nr:
        low = None
        low_bfac = 1000.0
        for s in bfact.atoms:
            if s.at_nr in lowest:
                continue
            if s.temp < low_bfac:
                low_bfac = s.temp
                low = s.copy()
        lowest[low.at_nr] = low        
    ret = []
    for k, v in lowest.iteritems():        
        ret.append(v)    
    writer = PDBWriter.PDBWriter()
    writer.writePDB("tmp_result_bfact.pdb", Protein.Protein(ret), False)
    reader = PDBReader.PDBReader()            
    sps = reader.readPDB("tmp_result_bfact.pdb", False)    
    filtered = CavityProbeIdentifier.filterProbesOutsideCavity(sps, nrOf, distTo)    
    writer.writePDB("tmp_result_bfact_filtered.pdb", Protein.Protein(filtered), False)
    cav_bfact = {}
    for a in filtered:        
        if a.res_nr not in cav_bfact:
            cav_bfact[a.res_nr] = (1, a.temp)
        if a.res_nr in cav_bfact:
            count, temp = cav_bfact[a.res_nr]
            count += 1
            temp += a.temp
            cav_bfact[a.res_nr] = (count, temp)
    ave_cav_temp = {}
    for k, v in cav_bfact.iteritems():
        count, temp = v        
        ave_cav_temp[k] = temp / float(count)
    for a in filtered:
        a.temp = ave_cav_temp[a.res_nr]     
    return Protein.Protein(filtered)    


def takeLowestFromBfactNoScaling(bfact, nrOf, distTo, perc=0.2):
    lowest = {}
    nr = int(len(bfact.atoms) * perc)        
    while len(lowest) < nr:
        low = None
        low_bfac = 1000.0
        for s in bfact.atoms:
            if s.at_nr in lowest:
                continue
            if s.temp < low_bfac:
                low_bfac = s.temp
                low = s.copy()
        lowest[low.at_nr] = low        
    ret = []
    for k, v in lowest.iteritems():        
        ret.append(v)    
    writer = PDBWriter.PDBWriter()
    writer.writePDB("tmp_result_bfact.pdb", Protein.Protein(ret), False)
    reader = PDBReader.PDBReader()            
    sps = reader.readPDB("tmp_result_bfact.pdb", False)    
    filtered = CavityProbeIdentifier.filterProbesOutsideCavity(sps, nrOf, distTo)    
    writer.writePDB("tmp_result_bfact_filtered.pdb", Protein.Protein(filtered), False)
#    cav_bfact = {}
#    for a in filtered:        
#        if a.res_nr not in cav_bfact:
#            cav_bfact[a.res_nr] = (1, a.temp)
#        if a.res_nr in cav_bfact:
#            count, temp = cav_bfact[a.res_nr]
#            count += 1
#            temp += a.temp
#            cav_bfact[a.res_nr] = (count, temp)
#    ave_cav_temp = {}
#    for k, v in cav_bfact.iteritems():
#        count, temp = v        
#        ave_cav_temp[k] = temp / float(count)
#    for a in filtered:
#        a.temp = ave_cav_temp[a.res_nr]     
    return Protein.Protein(filtered)


def putLowestOnProtein(lowest, prot):
    the_lowest_cavs = {}    
    deathcount = 0
    while len(the_lowest_cavs) < 3:
        deathcount += 1
        if deathcount > 10000:            
            break
        low = 1000.0
        low_nr = -1
        for a in lowest.atoms:
            if a.res_nr in the_lowest_cavs:
                continue
            if a.temp < low:
                low = a.temp
                low_nr = a.res_nr
        the_lowest_cavs[low_nr] = low
    used_sps = []    
    for a in lowest.atoms:
        if a.res_nr in the_lowest_cavs:
            used_sps.append(a)
#    print "anzahl der zu benutzenden sps: " + str(len(used_sps))
    for a in prot.atoms:
        for b in used_sps:
            if a.distToOtherNoSqrt(b) < 36.0:
                a.temp = b.temp                
    return prot

def putLargestOnProtein(lowest, prot):
    the_largest_cavs = {}
    cluster = {}
    for a in lowest.atoms:
        if a.res_nr not in cluster:
            cluster[a.res_nr] = 0
        cluster[a.res_nr] += 1
            
    deathcount = 0
    while len(the_largest_cavs) < 1:
        deathcount += 1
        if deathcount > 10000:            
            break
        size = 0
        cluster_nr = -1
        for k, v in cluster.iteritems():
            if k in the_largest_cavs:
                continue
            if v > size:
                size = v
                cluster_nr = k
        the_largest_cavs[cluster_nr] = size
    used_sps = []    
    for a in lowest.atoms:
        if a.res_nr in the_largest_cavs:
            used_sps.append(a)
#    print "anzahl der zu benutzenden sps: " + str(len(used_sps))
    for a in prot.atoms:
        for b in used_sps:
            if a.distToOtherNoSqrt(b) < 36.0:
                a.temp = b.temp                
    return prot


def getNeighboursOfLowest(cluster, bfact, cap):
    lowest = {}
    nr = int(len(bfact.atoms) / 5.0)
    if nr < 20:
        nr = 20
    if nr > len(bfact.atoms):
        nr = len(bfact.atoms)        
    while len(lowest) < nr:
        low = None
        low_bfac = 1000.0
        for s in bfact.atoms:
            if s.at_nr in lowest:
                continue
            if s.temp < low_bfac:
                low_bfac = s.temp
                low = s.copy()
        lowest[low.at_nr] = low        
    ret = []
    for k, v in lowest.iteritems():        
        ret.append(v)
    new = {}
    for i in range(len(ret)):
        ret[i].occ = float(i)            
    for b in ret:
        for a in cluster.atoms:
            if a.distToOtherNoSqrt(b) < cap:
                if b.occ not in new:
                    new[b.occ] = b.copy()
                break
    end = []
    for k, v in new.iteritems():
        end.append(v)
    return end            
            
