'''
Created on 19.11.2010

@author: sschneider
'''
from IO import PDBReader, PDBWriter
from Structure import Protein
import math

def writeElecDiffAsBfac(pdb, surf, probe_results, ref_elec):
    """
    Subtract from the desolvation values according to the calculation with given surface point the
    reference desolvation energy from the calculation without probes 
    """
    for a in surf.atoms:
        a.temp = probe_results[a.at_nr] - ref_elec
    writer = PDBWriter.PDBWriter()
    writer.writePDB(pdb + "_bfact.pdb", surf)
    return surf

def generateBindingSitePredictionSpotsSingle(surface_points_name):
    """
    Generates out of all surface points used for calculation of desolvation
    energy a given number of these surface points.
    Chosen are the ones with the lowest desolvation free energy not to close (2 times the 
    size of one imaginary patch) to one already chosen spot to avoid a large overlap.
    """
    reader = PDBReader.PDBReader()
    sps = reader.readPDB(surface_points_name, False)
    spotdist = 10.0
    result = []
    tmp = {}
    tmpSP = None
    tmpVal = 1000.0
    while len(tmp) < 6:
        for s in sps.atoms:
#            if int(s.segID) < 10:
#                continue
            if s.at_nr in tmp:
                continue
            inIt = False
            """
            skipping close ones
            """
            for k, v in tmp.iteritems():
                if v.distToOtherNoSqrt(s) < spotdist * spotdist * 2:
                    inIt = True
            if inIt == True:
                continue
            if s.temp < tmpVal:
                tmpVal = s.temp
                tmpSP = s.copy()
        if tmpSP == None:
            break
        tmp[tmpSP.at_nr] = tmpSP.copy()
        tmpSP = None
        tmpVal = 100.0
    """
    this time no marks set. marks will be set in method 
    putPredictionSpotOnProtein for defining prediction
    hot spots directly on the proteins surface.
    """
    for k, v in tmp.iteritems():
        result.append(v.copy())        
    return result


def putPredictionSpotOnProtein(spots, prot):
    """
    The calculated hot spots are transfered to the
    nearest atom on the protein surface. Than spots of given
    size (spotdist) are calculated marking the atoms belonging to 
    these hot spots with a 1.0 at the bfac (.temp) of the atom.
    """      
#    spotdist = 6.0
#    for a in prot.atoms:
#        a.temp = 0.0       
#    tmp_spots_on_prot = {}
#    for sp in spots:
#        index = -1
#        dist = 1000.0
#        for i in range(len(prot.atoms)):
#            d = distance(sp, prot.atoms[i])
#            if d < dist:
#                dist = d
#                index = i
#                temp = sp.temp
#        prot.atoms[index].temp = temp 
#        tmp_spots_on_prot[index] = prot.atoms[index]
#    for i, a in tmp_spots_on_prot.iteritems():
#        for at in prot.atoms:
#            if distance(a, at) < spotdist * spotdist:
#                at.temp = a.temp
#    surf_atoms_tmp = defineSurfaceAtoms(prot)
#    surf_atoms = {}
#    for a in surf_atoms_tmp:
#        surf_atoms[a.at_nr] = a    
#    for a in prot.atoms:
#        if a.at_nr not in surf_atoms:
#            a.temp = 0.0
#        else:
#            a.temp = 1.0
    for a in prot.atoms:
        a.temp = 0.0
    for sp in spots:
        for a in prot.atoms:
            if sp.distToOtherNoSqrt(a) < 36.0:
                a.temp = sp.temp
    return prot

def generateElecPatches(pdb, surfPoints_bfact, dist):
    """
    For each surface point (with desolvation energy mapped on the b-factor) calculate
    patches of nearest neighbours and do a simple totalValue/number calculation.
    For independence of patch values copy atoms are created.  
    """    
    external = []
    copy_atoms = [a.copy() for a in surfPoints_bfact.atoms]
    for a in surfPoints_bfact.atoms:
        count = 1        
        for b in copy_atoms:                    
            if a.at_nr != b.at_nr and math.sqrt(a.distToOtherNoSqrt(b)) < dist:                
                count += 1                
        if count > 0:
            external.append(a.copy())
    copy_atoms2 = [a.copy() for a in external]    
    for a in external:
        count = 1        
        for b in copy_atoms2:                    
            if a.at_nr != b.at_nr and math.sqrt(a.distToOtherNoSqrt(b)) < dist:
                a.temp += b.temp 
                count += 1
        a.temp /= count
        a.segID = str(count)        
    writer = PDBWriter.PDBWriter()
    writer.writePDB(pdb + "_Desolv_" + str(dist) + ".pdb", Protein.Protein(external), False)
    return Protein.Protein(external)
