/*
 * IO.cpp
 *
 *  Created on: 19.05.2009
 *      Author: sschneider
 */

#include "IO.h"

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <map>
#include <stdlib.h>
#include <math.h>

//***************************************************************
/*Reader*/
//***************************************************************


Reader::Reader() {

}

vector<string> Reader::tokenize(const string& str, char delim) {
	vector<string> tokens;
	stringstream mySstream(str);
	string temp;

	while (getline(mySstream, temp, delim))
		if (temp != "") {
			tokens.push_back(temp);
		}
	return tokens;
}

Protein Reader::readProtein(string filename) {
	cout << "reading in catched protein..." << endl;
	//datei einlesen
	ifstream fin(filename.c_str());
	string zeile;
	if (!fin) {
		cout << "file " << filename.c_str() << " does not exist" << endl;
		cout << "EXIT" << endl;
		exit(0);
	}
	//atome
	vector<Atom> atoms;

	while (getline(fin, zeile)) {
		vector<string> ele;
		ele = tokenize(zeile, ' ');
		if (ele[0] == "ATOM") {
			int nr = atoi(ele[1].c_str());
			string type = zeile.substr(13, 1);
			string acid = ele[3];
			int acidnr = atoi(ele[4].c_str());
			float x = float(atof(ele[5].c_str()));
			float y = float(atof(ele[6].c_str()));
			float z = float(atof(ele[7].c_str()));
			float occ = float(atof(ele[8].c_str()));
			float temp = float(atof(ele[9].c_str()));
			Atom a(nr, type, acid, acidnr, x, y, z, occ, temp);
			atoms.push_back(a);
		}
	}
	fin.close();
	Protein p("average", atoms);
	cout << "...done reading in average protein." << endl;
	return p;
}

vector<Point> Reader::readSurfacePoints(string filename) {
	cout << "reading in surface points..." << endl;
	//datei einlesen
	ifstream fin(filename.c_str());
	string zeile;
	if (!fin) {
		cout << "file " << filename.c_str() << " does not exist" << endl;
		cout << "EXIT" << endl;
		exit(0);
	}
	//atome
	vector<Point> atoms;
	//int count = 1;
	while (getline(fin, zeile)) {
		vector<string> ele;
		ele = tokenize(zeile, ' ');
		if (ele[0] == "ATOM") {
			//int nr = atoi(ele[1].c_str());

			int nr = atoi(ele[1].c_str());
			//			string acid = ele[3];
			int acid = atoi(ele[3].c_str());
			float x = float(atof(ele[6].c_str()));
			float y = float(atof(ele[7].c_str()));
			float z = float(atof(ele[8].c_str()));

			float temp = float(atof(ele[10].c_str()));

			Point a(x, y, z, nr, acid, temp);
			atoms.push_back(a);
		}
		//count++;
	}
	fin.close();
	cout << atoms.size() << " surface points read in" << endl;
	cout << "...done reading in surface points." << endl;
	return atoms;
}

//***************************************************************
/*WRITER*/
//***************************************************************

Writer::Writer() {
}

void Writer::writeProtein(vector<Atom> prot, string filename) {
	ofstream of(filename.c_str());
	//	vector<Atom> prot = average.atoms;

	for (unsigned int i = 0; i < prot.size(); i++) {
		Atom a = prot[i];
		if (a.type.size() <= 3) {

			of << setw(6) << left << "ATOM" << setw(5) << right << a.nr << "  " << setw(4) << left << a.type << setw(3)
					<< a.acid << setw(6) << right << a.acidnr << "     " << setw(6) << showpoint << fixed
					<< setprecision(3) << a.x << setw(8) << a.y << setw(8) << a.z << setprecision(2) << setw(6)
					<< right << a.occ << setw(5) << a.temp << endl;

		} else if (a.type.size() == 4) {

			of << setw(6) << left << "ATOM" << setw(5) << right << a.nr << " " << setw(4) << left << a.type << " "
					<< setw(3) << a.acid << setw(6) << right << a.acidnr << "     " << setw(6) << showpoint << fixed
					<< setprecision(3) << a.x << setw(8) << a.y << setw(8) << a.z << setprecision(2) << setw(6)
					<< right << a.occ << setw(5) << a.temp << endl;
		}
	}
	of.flush();
	of.close();
}

void Writer::writeSurfacePoints(vector<Point> sp, string filename) {
	std::ofstream of(filename.c_str());
	for (unsigned int i = 0; i < sp.size(); i++) {
		//Point a = sp[i];
		of << setw(6) << left << "ATOM" << setw(5) << right << i+1 << "  " << setw(4) << left << "O" << setw(3)
				<< "SPT" << setw(6) << right << sp[i].acid << "   " << setw(9) << showpoint << fixed
				<< setprecision(3) << sp[i].x << setw(8) << sp[i].y << setw(8) << sp[i].z << setprecision(2) << setw(6) << right
				<< 0.0 << setw(6) << sp[i].temp << endl;
	}
	of.flush();
	of.close();
}

