'''
Created on 03.06.2011

@author: sschneider
'''
from Structure import Atom, Protein


class PDBReader:
    """
    Class PDBReader provides methods to read in proteins in the PDB file format.  

    """
    err_str = "ERROR - Skipping this atom: A necessary part of the pdb file is missing! Chars: "
    warn_str = "WARNING - Setting default: A part of the pdb file is missing! Setting default for "
    
    def __init__(self):
        """
        No initialization
        """
        pass
    
    
    def readSingleAtomByLine(self, line):
        if line[0:6].strip() == "ATOM": 
            at_nr = ""
            at_name = ""
            rem_indic = ""
            branch_des = ""
            alt_loc = ""
            res_name = ""
            chain_id = ""
            res_nr = ""
            insertion_code = ""
            x = ""
            y = ""
            z = ""
            occ = ""
            segID = ""
            element = ""
            charge = ""               
            try:                
                at_nr = int(line[6:11].strip())
            except:
                print self.err_str + "6 to 10. Atom number in line " 
                return None               
            try:
                at_name = line[13].strip()
            except:
                print self.err_str + "12 to 15. Atom name in line " 
                return None
            try:
                rem_indic = line[14].strip()
            except:
                rem_indic = " "
            try:
                branch_des = line[15].strip()
            except:
                branch_des = " "
            try:
                alt_loc = line[16].strip()
            except:
                alt_loc = " "                
            try:
                res_name = line[17:20].strip()
            except:                
                res_name = " "
            try:
                chain_id = line[21].strip()
            except:
                chain_id = " "                
            try:
                res_nr = int(line[22:26].strip())
            except:
                res_nr = 0                
            try:
                insertion_code = line[26].strip()
            except:
                insertion_code = " "                
            try:
                x = float(line[30:38].strip())
            except:
                print self.err_str + "30 to 37. X position "
                return None
            try:
                y = float(line[38:46].strip())
            except:
                print self.err_str + "38 to 45. Y position \n"
                return None
            try:
                z = float(line[46:54].strip())
            except:
                print self.err_str + "46 to 53. Z position \n"
                return None
            try:
                occ = float(line[54:60].strip())
            except:
                occ = 0.0                
            try:
                temp = float(line[60:66].strip())
            except:
                temp = 0.0                
            try:
                segID = line[72:76].strip()
            except:
                segID = " "                
            try:
                element = line[76:78].strip()
            except:
                element = " "                
            try:
                charge = line[78:80].strip()
            except:
                charge = " "                            
            return Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge)            
   
    def readSingleHetatomByLine(self, line):
        if line[0:6].strip().find("HET") != -1: 
            at_nr = ""
            at_name = ""
            rem_indic = ""
            branch_des = ""
            alt_loc = ""
            res_name = ""
            chain_id = ""
            res_nr = ""
            insertion_code = ""
            x = ""
            y = ""
            z = ""
            occ = ""
            segID = ""
            element = ""
            charge = ""               
            try:                
                at_nr = int(line[6:11].strip())
            except:
                print self.err_str + "6 to 10. Atom number in line " 
                return None               
            try:
                at_name = line[13].strip()
            except:
                print self.err_str + "12 to 15. Atom name in line " 
                return None
            try:
                rem_indic = line[14].strip()
            except:
                rem_indic = " "
            try:
                branch_des = line[15].strip()
            except:
                branch_des = " "
            try:
                alt_loc = line[16].strip()
            except:
                alt_loc = " "                
            try:
                res_name = line[17:20].strip()
            except:                
                res_name = " "
            try:
                chain_id = line[21].strip()
            except:
                chain_id = " "                
            try:
                res_nr = int(line[22:26].strip())
            except:
                res_nr = 0                
            try:
                insertion_code = line[26].strip()
            except:
                insertion_code = " "                
            try:
                x = float(line[30:38].strip())
            except:
                print self.err_str + "30 to 37. X position "
                return None
            try:
                y = float(line[38:46].strip())
            except:
                print self.err_str + "38 to 45. Y position \n"
                return None
            try:
                z = float(line[46:54].strip())
            except:
                print self.err_str + "46 to 53. Z position \n"
                return None
            try:
                occ = float(line[54:60].strip())
            except:
                occ = 0.0                
            try:
                temp = float(line[60:66].strip())
            except:
                temp = 0.0                
            try:
                segID = line[72:76].strip()
            except:
                segID = " "                
            try:
                element = line[76:78].strip()
            except:
                element = " "                
            try:
                charge = line[78:80].strip()
            except:
                charge = " "                            
            return Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge)
   
    def readPDB(self, filename, verbose=True):     
        """
        Reads in a PDB file from the given filename and returns a Protein 
        object. The verbose flag indicates if warnings should be printed on 
        the stdout. \n
        Atoms will be skipped if one of the following data is not readable: \n
        a) atom number (char 6 to 10) \n
        b) atom name (char 12 to 15) \n
        c) x, y or z position (char 30 to 53) \n
        Error lines are be printed out.
        
        If one of the other informations needed to create an atom is missing 
        defaults will be set. (printed out if verbose == True)   
        
        @param filename: The filename of the PDB file. 
        @type filename: string
        @param verbose: Flag for printing out warnings
        @type verbose: boolean 
        
        @return: A Protein or None if file could not be read or contained no suitable data.
        """           
        # file operations
        try:
            myfile = open(filename)
        except:
#            print
#            print "ERROR: File not found."
#            print "---> Could not find file " + filename
#            print "---> make sure it is spelled correctly and in the right folder."
#            print
#            print "returning None"            
            return None
        if verbose == True:
            print
            print "Reading in file " + filename + "..."
            print
        lines = myfile.readlines()
        # list of atoms to create protein
        atoms = []
        # error and warning strings
    
        # log file
        log = []
        # reading file line by line
        linecount = 0
        for line in lines:         
            linecount += 1
            #only reads if it is an atom                           
            if line[0:6].strip() == "ATOM": 
                at_nr = ""
                at_name = ""
                rem_indic = ""
                branch_des = ""
                alt_loc = ""
                res_name = ""
                chain_id = ""
                res_nr = ""
                insertion_code = ""
                x = ""
                y = ""
                z = ""
                occ = ""
                segID = ""
                element = ""
                charge = ""               
                try:                
                    at_nr = int(line[6:11].strip())
                except:
                    print self.err_str + "6 to 10. Atom number in line " + str(linecount)
                    continue                  
                try:
                    at_name = line[13].strip()
                except:
                    print self.err_str + "12 to 15. Atom name in line " + str(linecount)
                    continue
                try:
                    rem_indic = line[14].strip()
                except:
                    rem_indic = " "
                try:
                    branch_des = line[15].strip()
                except:
                    branch_des = " "
                try:
                    alt_loc = line[16].strip()
                except:
                    alt_loc = " "
                    log.append(self.warn_str + "alternate location \n")
                try:
                    res_name = line[17:20].strip()
                except:
                    log.append(self.warn_str + "residue name \n")
                    res_name = " "
                try:
                    chain_id = line[21].strip()
                except:
                    chain_id = " "
                    log.append(self.warn_str + "chain ID \n")
                try:
                    res_nr = int(line[22:26].strip())
                except:
                    res_nr = 0
                    log.append(self.warn_str + "residue number \n")
                try:
                    insertion_code = line[26].strip()
                except:
                    insertion_code = " "
                    log.append(self.warn_str + "insertion code \n")
                try:
                    x = float(line[30:38].strip())
                except:
#                    print self.err_str + "30 to 37. X position "
#                    print filename
                    return None
                try:
                    y = float(line[38:46].strip())
                except:
#                    print self.err_str + "38 to 45. Y position \n"
##                    continue
#                    print filename
                    return None
                try:
                    z = float(line[46:54].strip())
                except:
#                    print self.err_str + "46 to 53. Z position \n"
#                    continue
#                    print filename
                    return None
                try:
                    occ = float(line[54:60].strip())
                except:
                    occ = 0.0
                    log.append(self.warn_str + "occupancy \n")
                try:
                    temp = float(line[60:66].strip())
                except:                    
                    temp = 0.0
                    log.append(self.warn_str + "temperature \n")
                try:
                    segID = line[72:76].strip()
                except:
                    segID = " "
                    log.append(self.warn_str + "segment identifier \n")
                try:
                    element = line[76:78].strip()
                except:
                    element = " "
                    log.append(self.warn_str + "element symbol \n")
                try:
                    charge = line[78:80].strip()
                except:
                    charge = " "
                    log.append(self.warn_str + "charge \n")            
                
                atoms.append(Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge))
            elif line[0:6].strip() == "ENDMDL":
                break
        if verbose == True and len(log) > 0:
            print "".join(log)
        if len(atoms) > 0:
            if verbose == True:
                print "...done reading in PDB"
            return Protein.Protein(atoms)
        else:
            print "ERROR: No atoms read, no protein created. Returning None."
            return None
        
    def readLigand(self, filename, verbose=True):     
        """
        Reads in a PDB file from the given filename and returns a Protein 
        object. The verbose flag indicates if warnings should be printed on 
        the stdout. \n
        Atoms will be skipped if one of the following data is not readable: \n
        a) atom number (char 6 to 10) \n
        b) atom name (char 12 to 15) \n
        c) x, y or z position (char 30 to 53) \n
        Error lines are be printed out.
        
        If one of the other informations needed to create an atom is missing 
        defaults will be set. (printed out if verbose == True)   
        
        @param filename: The filename of the PDB file. 
        @type filename: string
        @param verbose: Flag for printing out warnings
        @type verbose: boolean 
        
        @return: A Protein or None if file could not be read or contained no suitable data.
        """           
        # file operations
#        try:
        myfile = open(filename)
#        except:
#            print
#            print "ERROR: File not found."
#            print "---> Could not find file " + filename
#            print "---> make sure it is spelled correctly and in the right folder."
#            print
#            print "returning None"            
#            return None
#        if verbose == True:
#            print
#            print "Reading in file " + filename + "..."
#            print
        lines = myfile.readlines()
        # list of atoms to create protein
        atoms = []
        # error and warning strings
    
        # log file
        log = []
        # reading file line by line
        linecount = 0
        for line in lines:         
            linecount += 1
            #only reads if it is an atom                           
            if line[0:2].strip() == "HE": 
                at_nr = ""
                at_name = ""
                rem_indic = ""
                branch_des = ""
                alt_loc = ""
                res_name = ""
                chain_id = ""
                res_nr = ""
                insertion_code = ""
                x = ""
                y = ""
                z = ""
                occ = ""
                segID = ""
                element = ""
                charge = ""               
                try:                
                    at_nr = int(line[6:11].strip())
                except:
                    print self.err_str + "6 to 10. Atom number in line " + str(linecount)
                    continue                  
                try:
                    at_name = line[13].strip()
                except:
                    print self.err_str + "12 to 15. Atom name in line " + str(linecount)
                    continue
                try:
                    rem_indic = line[14].strip()
                except:
                    rem_indic = " "
                try:
                    branch_des = line[15].strip()
                except:
                    branch_des = " "
                try:
                    alt_loc = line[16].strip()
                except:
                    alt_loc = " "
                    log.append(self.warn_str + "alternate location \n")
                try:
                    res_name = line[17:20].strip()
                except:
                    log.append(self.warn_str + "residue name \n")
                    res_name = " "
                try:
                    chain_id = line[21].strip()
                except:
                    chain_id = " "
                    log.append(self.warn_str + "chain ID \n")
                try:
                    res_nr = int(line[22:26].strip())
                except:
                    res_nr = 0
                    log.append(self.warn_str + "residue number \n")
                try:
                    insertion_code = line[26].strip()
                except:
                    insertion_code = " "
                    log.append(self.warn_str + "insertion code \n")
                try:
                    x = float(line[30:38].strip())
                except:
                    print self.err_str + "30 to 37. X position "
                    continue
                try:
                    y = float(line[38:46].strip())
                except:
                    print self.err_str + "38 to 45. Y position \n"
                    continue
                try:
                    z = float(line[46:54].strip())
                except:
                    print self.err_str + "46 to 53. Z position \n"
                    continue
                try:
                    occ = float(line[54:60].strip())
                except:
                    occ = 0.0
                    log.append(self.warn_str + "occupancy \n")
                try:
                    temp = float(line[60:66].strip())
                except:                    
                    temp = 0.0
                    log.append(self.warn_str + "temperature \n")
                try:
                    segID = line[72:76].strip()
                except:
                    segID = " "
                    log.append(self.warn_str + "segment identifier \n")
                try:
                    element = line[76:78].strip()
                except:
                    element = " "
                    log.append(self.warn_str + "element symbol \n")
                try:
                    charge = line[78:80].strip()
                except:
                    charge = " "
                    log.append(self.warn_str + "charge \n")            
                
                atoms.append(Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge))
            elif line[0:6].strip() == "ENDMDL":
                break
        myfile.close()
        if verbose == True and len(log) > 0:
            print "".join(log)
        if len(atoms) > 0:
            if verbose == True:
                print "...done reading in PDB"
            return Protein.Protein(atoms)
        else:
            print "ERROR: No atoms read, no protein created. Returning None."
            return None
     
     
    def readMultipleLigands(self, filename, protFilename, verbose=True):     
        """
        Reads in a PDB file from the given filename and returns a Protein 
        object. The verbose flag indicates if warnings should be printed on 
        the stdout. \n
        Atoms will be skipped if one of the following data is not readable: \n
        a) atom number (char 6 to 10) \n
        b) atom name (char 12 to 15) \n
        c) x, y or z position (char 30 to 53) \n
        Error lines are be printed out.
        
        If one of the other informations needed to create an atom is missing 
        defaults will be set. (printed out if verbose == True)   
        
        @param filename: The filename of the PDB file. 
        @type filename: string
        @param verbose: Flag for printing out warnings
        @type verbose: boolean 
        
        @return: A Protein or None if file could not be read or contained no suitable data.
        """           
        # file operations        
        prot = self.readPDB(protFilename, False)
        myfile = open(filename)
        
        lines = myfile.readlines()
        # list of atoms to create protein
        ligands = {}
        atoms = []
        # error and warning strings
    
        # log file
        log = []
        # reading file line by line
        linecount = 0
        nr = 1
        for line in lines:         
            linecount += 1
            #only reads if it is an atom                           
            if line[0:2].strip() == "HE": 
                at_nr = ""
                at_name = ""
                rem_indic = ""
                branch_des = ""
                alt_loc = ""
                res_name = ""
                chain_id = ""
                res_nr = ""
                insertion_code = ""
                x = ""
                y = ""
                z = ""
                occ = ""
                segID = ""
                element = ""
                charge = ""               
                try:                
                    at_nr = int(line[6:11].strip())
                except:
                    print self.err_str + "6 to 10. Atom number in line " + str(linecount)
                    continue                  
                try:
                    at_name = line[13].strip()
                except:
                    print self.err_str + "12 to 15. Atom name in line " + str(linecount)
                    continue
                try:
                    rem_indic = line[14].strip()
                except:
                    rem_indic = " "
                try:
                    branch_des = line[15].strip()
                except:
                    branch_des = " "
                try:
                    alt_loc = line[16].strip()
                except:
                    alt_loc = " "
                    log.append(self.warn_str + "alternate location \n")
                try:
                    res_name = line[17:20].strip()
                except:
                    log.append(self.warn_str + "residue name \n")
                    res_name = " "
                try:
                    chain_id = line[21].strip()
                except:
                    chain_id = " "
                    log.append(self.warn_str + "chain ID \n")
                try:
                    res_nr = int(line[22:26].strip())
                except:
                    res_nr = 0
                    log.append(self.warn_str + "residue number \n")
                try:
                    insertion_code = line[26].strip()
                except:
                    insertion_code = " "
                    log.append(self.warn_str + "insertion code \n")
                try:
                    x = float(line[30:38].strip())
                except:
                    print self.err_str + "30 to 37. X position "
                    continue
                try:
                    y = float(line[38:46].strip())
                except:
                    print self.err_str + "38 to 45. Y position \n"
                    continue
                try:
                    z = float(line[46:54].strip())
                except:
                    print self.err_str + "46 to 53. Z position \n"
                    continue
                try:
                    occ = float(line[54:60].strip())
                except:
                    occ = 0.0
                    log.append(self.warn_str + "occupancy \n")
                try:
                    temp = float(line[60:66].strip())
                except:                    
                    temp = 0.0
                    log.append(self.warn_str + "temperature \n")
                try:
                    segID = line[72:76].strip()
                except:
                    segID = " "
                    log.append(self.warn_str + "segment identifier \n")
                try:
                    element = line[76:78].strip()
                except:
                    element = " "
                    log.append(self.warn_str + "element symbol \n")
                try:
                    charge = line[78:80].strip()
                except:
                    charge = " "
                    log.append(self.warn_str + "charge \n")            
                a = Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge)
                atoms.append(a)
#                for b in prot.atoms:
#                    if ((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z)) < 25.0:
#                        atoms.append(a)
            elif line[0:6].strip() == "TER":
                if len(atoms) > 0:
                    ligands[nr] = ([a.copy() for a in atoms])
                    atoms[:] = []
                    nr += 1    
        myfile.close()
        if len(ligands) == 0 and len(atoms) > 0:
            ligands[nr] = atoms
        if len(ligands) > 0:            
            return ligands
        else:
            print "ERROR: No atoms read, no protein created. Returning None."
            return None    
        
    def readMol(self, filename):
        lines = open(filename).readlines()
        atomsec = False
        atoms = []
        for line in lines:            
            ele = line.split()
            if len(ele) < 1:
                continue
            if ele[0] == "@<TRIPOS>ATOM":
                atomsec = True
                continue
            if ele[0] == "@<TRIPOS>BOND":
                atomsec = False
            if atomsec == True:
                at_nr = int(ele[0])
                at_name = ele[1][0]
                x = float(ele[2])
                y = float(ele[3])
                z = float(ele[4])
                res_name = ele[7][0:3]
                res_nr = int(ele[6])
                atoms.append(Atom.Atom(at_nr, at_name, "", "", "", res_name, "", res_nr, "", x, y, z, 0.0, 0.0, "", "", ""))
        return Protein.Protein(atoms)
    
    def readSingleGridAtomByLine(self, line):
        if line[0:6].strip() == "ATOM": 
            at_nr = ""
            at_name = ""
            rem_indic = ""
            branch_des = ""
            alt_loc = ""
            res_name = ""
            chain_id = ""
            res_nr = ""
            insertion_code = ""
            x = ""
            y = ""
            z = ""
            occ = ""
            segID = ""
            element = ""
            charge = ""               
            try:                
                at_nr = int(line[6:11].strip())
            except:
                print self.err_str + "6 to 10. Atom number in line " 
                return None               
            try:
                at_name = line[13].strip()
            except:
                print self.err_str + "12 to 15. Atom name in line " 
                return None
            try:
                rem_indic = line[14].strip()
            except:
                rem_indic = " "
            try:
                branch_des = line[15].strip()
            except:
                branch_des = " "
            try:
                alt_loc = line[16].strip()
            except:
                alt_loc = " "                
            try:
                res_name = line[17:20].strip()
            except:                
                res_name = " "
            try:
                chain_id = line[21].strip()
            except:
                chain_id = " "                
            try:
                res_nr = int(line[22:26].strip())
            except:
                res_nr = 0                
            try:
                insertion_code = line[26].strip()
            except:
                insertion_code = " "                
            try:
                x = float(line[30:38].strip())
            except:
                print self.err_str + "30 to 37. X position "
                return None
            try:
                y = float(line[38:46].strip())
            except:
                print self.err_str + "38 to 45. Y position \n"
                return None
            try:
                z = float(line[46:54].strip())
            except:
                print self.err_str + "46 to 53. Z position \n"
                return None
            try:
                occ = float(line[54:60].strip())
            except:
                occ = 0.0                
            try:
                temp = float(line[60:70].strip())
            except:
                temp = 0.0                
#            try:
#                segID = line[72:76].strip()
#            except:
#                segID = " "                
#            try:
#                element = line[76:78].strip()
#            except:
#                element = " "                
#            try:
#                charge = line[78:80].strip()
#            except:
#                charge = " "                            
            return Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge)
        
        
    def readGrid(self, filename, verbose=True):     
        """
        Reads in a PDB file from the given filename and returns a Protein 
        object. The verbose flag indicates if warnings should be printed on 
        the stdout. \n
        Atoms will be skipped if one of the following data is not readable: \n
        a) atom number (char 6 to 10) \n
        b) atom name (char 12 to 15) \n
        c) x, y or z position (char 30 to 53) \n
        Error lines are be printed out.
        
        If one of the other informations needed to create an atom is missing 
        defaults will be set. (printed out if verbose == True)   
        
        @param filename: The filename of the PDB file. 
        @type filename: string
        @param verbose: Flag for printing out warnings
        @type verbose: boolean 
        
        @return: A Protein or None if file could not be read or contained no suitable data.
        """           
        # file operations
        try:
            myfile = open(filename)
        except:
            print
            print "ERROR: File not found."
            print "---> Could not find file " + filename
            print "---> make sure it is spelled correctly and in the right folder."
            print
            print "returning None"            
            return None
        if verbose == True:
            print
            print "Reading in file " + filename + "..."
            print
        lines = myfile.readlines()
        # list of atoms to create protein
        atoms = []
        # error and warning strings
    
        # log file
        log = []
        # reading file line by line
        linecount = 0
        for line in lines:         
            linecount += 1
            #only reads if it is an atom                           
            if line[0:6].strip() == "ATOM": 
                at_nr = ""
                at_name = ""
                rem_indic = ""
                branch_des = ""
                alt_loc = ""
                res_name = ""
                chain_id = ""
                res_nr = ""
                insertion_code = ""
                x = ""
                y = ""
                z = ""
                occ = ""
                segID = ""
                element = ""
                charge = ""               
                try:                
                    at_nr = int(line[6:11].strip())
                except:
                    print self.err_str + "6 to 10. Atom number in line " + str(linecount)
                    continue                  
                try:
                    at_name = line[13].strip()
                except:
                    print self.err_str + "12 to 15. Atom name in line " + str(linecount)
                    continue
                try:
                    rem_indic = line[14].strip()
                except:
                    rem_indic = " "
                try:
                    branch_des = line[15].strip()
                except:
                    branch_des = " "
                try:
                    alt_loc = line[16].strip()
                except:
                    alt_loc = " "
                    log.append(self.warn_str + "alternate location \n")
                try:
                    res_name = line[17:20].strip()
                except:
                    log.append(self.warn_str + "residue name \n")
                    res_name = " "
                try:
                    chain_id = line[21].strip()
                except:
                    chain_id = " "
                    log.append(self.warn_str + "chain ID \n")
                try:
                    res_nr = int(line[22:26].strip())
                except:
                    res_nr = 0
                    log.append(self.warn_str + "residue number \n")
                try:
                    insertion_code = line[26].strip()
                except:
                    insertion_code = " "
                    log.append(self.warn_str + "insertion code \n")
                try:
                    x = float(line[30:38].strip())
                except:
                    return None
                try:
                    y = float(line[38:46].strip())
                except:
                    return None
                try:
                    z = float(line[46:54].strip())
                except:
                    return None
                try:
                    occ = float(line[54:60].strip())
                except:
                    occ = 0.0
                    log.append(self.warn_str + "occupancy \n")
                try:
                    temp = float(line[60:70].strip())
                except:                    
                    temp = 0.0
                    log.append(self.warn_str + "temperature \n")           
                
                atoms.append(Atom.Atom(at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge))
            elif line[0:6].strip() == "ENDMDL":
                break
        if verbose == True and len(log) > 0:
            print "".join(log)
        if len(atoms) > 0:
            if verbose == True:
                print "...done reading in PDB"
            return Protein.Protein(atoms)
        else:
            print "ERROR: No atoms read, no protein created. Returning None."
            return None