'''
Created on 03.06.2011

@author: sschneider
'''
class PDBWriter:
    """
    Class PDBWriter provides methods to write Molecule.Protein proteins in the PDB file format.
    
    Usage:
    
    
    import PDB_IO \n
    from Molecule import Protein
    
    [...]
        
    writy = PDB_IO.PDBWriter() \n
    writy.writePDB("my.pdb", myprotein)
    """

    
    def __init__(self):
        """
        No initialization
        """
        pass
    
    def writePDB(self, filename, protein, verbose=True):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        if verbose == True:
            print
            print "try to write protein to file: " + str(filename) + "..."
            print
        atoms = protein.atoms        
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        lastNr = -1
        for a in atoms:
            try:        
                outline = str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s \n" 
                                  % ("ATOM", a.at_nr, a.at_name, a.rem_indic, a.branch_des, a.alt_loc, a.res_name,
                                    a.chain_id, a.res_nr, a.insertion_code, a.x, a.y, a.z, a.occ, a.temp, a.segID,
                                    a.element, a.charge))     
                outFile.write(outline)
                if lastNr == -1:
                    lastNr = a.res_nr
                else:
                    if a.res_nr - lastNr > 1:
                        outFile.write("TER \n")                        
                    if lastNr < a.res_nr:
                        lastNr = a.res_nr
#                    continue   
                if (outline).find("OXT") != -1:
                    outFile.write("TER \n")    
            except:
                print
                print "ERROR: Could not write file"
                print "Check if you have permissions to write."
                print "Check disk quota."
                return
        outFile.write("END \n")
        outFile.close()
#        import time
#        time.sleep(1)
  
  
     
        
    '''
    Write methods for desolvation prediction or wherever ones needs Point3D surfacePoints
    '''
    
    
    def writePDBForCppSP(self, filename, protein, verbose=True):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        if verbose == True:
            print
            print "try to write protein to file: " + str(filename) + "..."
            print
        atoms = protein.atoms        
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        for a in atoms:                 
            try:
                outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                  % ("ATOM", a.at_nr, a.at_name, "", "", "", a.res_name,
                                    "", a.res_nr, "", a.x, a.y, a.z, a.occ, a.temp, "",
                                    "", "")))
                outFile.write("\n")
                #print "wrote protein to file: " + filename
            except:
                print
                print "ERROR: Could not write file"
                print "Check if you have permissions to write."
                print "Check disk quota."
                return        
        outFile.close()
        
         
    def writeSP(self, filename, atoms):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
#        print
#        print "try to write protein to file: " + str(filename) + "..."
#        print                
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        count = 1
        for a in atoms:
#            outvec.append(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
#                              % ("ATOM", a.at_nr, a.at_name, a.rem_indic, a.branch_des, a.alt_loc, a.res_name,
#                                a.chain_id, a.res_nr, a.insertion_code, a.x, a.y, a.z, a.occ, a.temp, a.segID,
#                                a.element, a.charge)))
                 
            try:
                outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%10.6f " 
                                  % ("ATOM", count, "H", "", "", "", "SPS",
                                    "", a.res_nr, "", a.x, a.y, a.z, a.occ, a.temp)))
                outFile.write("\n")
#                count += 1
                #print "wrote protein to file: " + filename
            except:
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%10.6f" 
                                  % ("ATOM", count, "H", "", "", "", "SPS",
                                    "", a.res_nr, "", a.x, a.y, a.z, a.occ, a.temp)))
                    outFile.write("\n")
#                    count += 1
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return          
        outFile.close()
        
        
    def writeSPatches(self, filename, patches):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        print
        print "try to write protein to file: " + str(filename) + "..."
        print                
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        modell = 1
        for patch in patches:
            outFile.write("MODELL " + str(modell) + "\n")
            modell += 1
#            count = 1
            a = patch.center
            try:
                outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                  % ("ATOM", a.nr, "O", "", "", "", "SPS",
                                    "", a.res_nr, "", a.x, a.y, a.z, 0.00, a.temp, "",
                                    "", "")))
                outFile.write("\n")                
            except:
                print
                print "ERROR: Could not write file"
                print "Check if you have permissions to write."
                print "Check disk quota."
                return  
            for a in patch.points:                
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", a.nr, "O", "", "", "", "SPS",
                                        "", a.res_nr, "", a.x, a.y, a.z, 0.00, a.temp, "",
                                        "", "")))
                    outFile.write("\n")                            
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return  
            outFile.write("END \n")        
        outFile.close() 

    def writeSPatchesNoModel(self, filename, patches):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        print
        print "try to write protein to file: " + str(filename) + "..."
        print                
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        modell = 1
        for patch in patches:                        
#            count = 1
            a = patch.center
            try:
                outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                  % ("ATOM", a.nr, "O", "", "", "", "SPS",
                                    "", a.res_nr, "", a.x, a.y, a.z, 0.00, a.temp, "",
                                    "", "")))
                outFile.write("\n")                
            except:
                print
                print "ERROR: Could not write file"
                print "Check if you have permissions to write."
                print "Check disk quota."
                return  
            for a in patch.points:                
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", a.nr, "O", "", "", "", "SPS",
                                        "", a.res_nr, "", a.x, a.y, a.z, 0.00, a.temp, "",
                                        "", "")))
                    outFile.write("\n")                            
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return        
        outFile.close()
        
        
        
    def writeSPatchesHull(self, filename, patches):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        print
        print "try to write protein to file: " + str(filename) + "..."
        print                
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")        
        count = 1
        for patch in patches:                                
            for a in patch:                
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", count, "O", "", "", "", "SPS",
                                        "", a.res_nr, "", a.x, a.y, a.z, 0.00, 0.00, "",
                                        "", "")))
                    outFile.write("\n")
                    count += 1                
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return                  
        outFile.close()               
            
    def writeSPatchesWithProtein(self, filename, patches, pdb):        
        """
        This method writes atoms in a file respecting the PDB 2.0 format.
        @param filename: the name of the file for writing in
        @type filename: string
        @param protein: a protein
        @type protein: Object Molecule.Protein        
        """
        print
        print "try to write protein to file: " + str(filename) + "..."
        print                
        outFile_beg = file(filename, "wb")
        outFile_beg.write("")
        outFile = file(filename, "wa")
        modell = 1
        for patch in patches:
            outFile.write("MODELL " + str(modell) + "\n")
            modell += 1
            count = 1
            atom_count = 1
            for a in pdb.atoms:
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                  % ("ATOM", a.at_nr, a.at_name, a.rem_indic, a.branch_des, a.alt_loc, a.res_name,
                                    a.chain_id, a.res_nr, a.insertion_code, a.x, a.y, a.z, a.occ, a.temp, a.segID,
                                    a.element, a.charge)))
                    outFile.write("\n") 
                    atom_count += 1
                #print "wrote protein to file: " + filename
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return
            outFile.write("TER \n")
            for a in patch:                
                try:
                    outFile.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", count, "O", "", "", "", "SPS",
                                        "", a.res_nr, "", a.x, a.y, a.z, 0.00, 0.00, "",
                                        "", "")))
                    outFile.write("\n")
                    count += 1                
                except:
                    print
                    print "ERROR: Could not write file"
                    print "Check if you have permissions to write."
                    print "Check disk quota."
                    return  
            outFile.write("END \n")        
        outFile.close()  
