'''
Created on 17.11.2010

@author: sschneider
'''
import os, sys
import Data_Handling
from Analyzes import SinglePointAnalyzes
from SurfPointCreator import SinglePointsInCavityGenerator
from IO import PDBReader, PDBWriter
from Structure import  Protein

class APBS:
    
    pdb2pqr = "pdb2pqr.py"
    apbs="apbs"
    single = True
    probesize = 1.4
        
    def __init__(self, pdb, probesize, cs1, distFac):
        self.pdb = pdb        
        self.probesize = probesize
        self.cs1 = cs1
        self.distFac = distFac        
    
    def addSurfacePointToPQR(self, probe, sp):
        tmp_pqr = open(self.pdb + ".pqr")
        tmp_pqr_lines = tmp_pqr.readlines()
        tmp_pqr_lines.append("TER \n")
        tmp_pqr_lines.append("%-30s%8.3f%8.3f%8.3f%-9s%6.4f" % ("ATOM   9999  P   PRO   999    ", sp.x, sp.y, sp.z, "  0.0000 ", probe))        
        tmp_pqr_lines.append("\n")
        tmp_pqr_lines.append("TER \n")
        tmp_pqr_lines.append("END \n")
        tmp_pqr.close()
        tmp_pqr_write = file(self.pdb + "_tmp.pqr", "w")
        tmp_pqr_write.write("".join(tmp_pqr_lines))
        tmp_pqr_write.close()
        return self.pdb + "_tmp.pqr"
    
    def addMultipleSurfacePointsToPQR(self, patch, probe):
        tmp_pqr = open(self.pdb + ".pqr")
        tmp_pqr_lines = tmp_pqr.readlines()
        tmp_pqr_lines.append("TER \n")
        tmp_pqr_lines.append("%-30s%8.3f%8.3f%8.3f%-9s%6.4f" % ("ATOM   9999  P   PRO   999    ", patch.center.x, patch.center.y, patch.center.z, "  0.0000 ", probe))        
        tmp_pqr_lines.append("\n")
        for sp in patch.points:
            tmp_pqr_lines.append("%-30s%8.3f%8.3f%8.3f%-9s%6.4f" % ("ATOM   9999  P   PRO   999    ", sp.x, sp.y, sp.z, "  0.0000 ", probe))        
            tmp_pqr_lines.append("\n")
        tmp_pqr_lines.append("TER \n")
        tmp_pqr_lines.append("END \n")
        tmp_pqr.close()
        tmp_pqr_write = file(self.pdb + "_tmp.pqr", "w")
        tmp_pqr_write.write("".join(tmp_pqr_lines))
        tmp_pqr_write.close()
        return self.pdb + "_tmp.pqr"
    
        
    def getBoxSize(self, tmp_prot):
        xHigh = -1000.0
        xLow = 10000.0
        yHigh = -1000.0
        yLow = 10000.0
        zHigh = -1000.0
        zLow = 10000.0
        
        for a in tmp_prot.atoms:    
            if a.x - a.vdw < xLow:
                xLow = a.x - a.vdw
            if a.x + a.vdw > xHigh:
                xHigh = a.x + a.vdw 
            if a.y - a.vdw < yLow:
                yLow = a.y - a.vdw
            if a.y + a.vdw > yHigh:
                yHigh = a.y + a.vdw
            if a.z - a.vdw < zLow:
                zLow = a.z - a.vdw
            if a.z + a.vdw > zHigh:
                zHigh = a.z + a.vdw
        x = xHigh - xLow
        y = yHigh - yLow
        z = zHigh - zLow
        box = [(x, y, z), (x * 2.0, y * 2.0, z * 2.0), (x + 2.0, y + 2.0, z + 2.0)]
        return box
    
    def apbsSingleProbeRun(self, sp, bp, reuse):
        
        """
        generate box:
        - solvate.dat         
        - get box data 
        """
        
        """
        TEST correction
        """
        Data_Handling.createSolvDatNew()                

        acceptable = {"ALA":0, "ARG":0, "ASN":0, "ASP":0, "CYS":0, "GLU":0, "GLN":0, "GLY":0, "HIS":0, "ILE":0, "LEU":0, "LYS":0, "MET":0, "PHE":0, "PRO":0, "SER":0, "THR":0, "TRP":0, "TYR":0, "VAL":0}
        """
        TEST correction
        """
        read = PDBReader.PDBReader()
        print self.pdb,
        cor_prot = read.readPDB(self.pdb + ".pdb", False)
        new_cor_prot = []
        for a in cor_prot.atoms:
            if a.vdw > 0.0 and a.res_name in acceptable:
                new_cor_prot.append(a.copy())  
        Data_Handling.createTleapForAmberGB(self.pdb, True)           
        wr = PDBWriter.PDBWriter()
        wr.writePDB(self.pdb + ".pdb", Protein.Protein(new_cor_prot))
        os.system("tleap -f " + self.pdb + ".tleap > muell.txt")                        
        
        
        Data_Handling.createSolvDatNew()
        sic = SinglePointsInCavityGenerator.ProbesInCavity(self.pdb)
        nsurf, surf = sic.generateProbesInCavity(sp, bp, self.cs1, self.distFac, reuse)            

        reader = PDBReader.PDBReader()
        tmp_prot = reader.readPDB(self.pdb + ".pdb")
        box = self.getBoxSize(tmp_prot)
        """
        fix file with pdb2pqr:
        - run pdb2pqr        
        """  
        os.system(self.pdb2pqr + " --ff=amber " + self.pdb + ".pdb " + self.pdb + ".pqr")        
        
        """
        bp calculations preperation
        - generate input file for reference calculation
        - calculate reference structure with apbs
        - grep elec energy and write to file
        """
        Data_Handling.createReferenceInputFileForAPBS(self.pdb, box)
        os.system(self.apbs + " " + self.pdb + ".in > " + self.pdb + ".out")
        elec = Data_Handling.getElecFromAPBS(self.pdb + ".out")
        tmp_f = file(self.pdb + "_ref.out", "w")
        tmp_f.write(self.pdb + ".out " + str(elec))
        tmp_f.close()    
      
        probe_results = {}
        """
       generate probes                       
        """
        
        """
        step bp calculations for probes
        - read in / generate probes
        - create pqr file for elec calc
        - call apbs 
        - gather results
        """
        Data_Handling.createTemporaryInputFileForAPBS(self.pdb, box)
        for i in range(len(surf.atoms)):     
            print "calc " + str(i) + " of " + str(nsurf),
            sys.stdout.flush()
            a = surf.atoms[i]               
            self.addSurfacePointToPQR(self.probesize, a)            
            os.system(self.apbs + " " + self.pdb + "_tmp.in > " + self.pdb + "_tmp.out")
#            probe_results[a.at_nr] = Data_Handling.getElecFromAPBS(self.pdb + "_tmp.out")    
            elec_tmp = Data_Handling.getElecFromAPBS(self.pdb + "_tmp.out")
            if elec_tmp != None:
                probe_results[a.at_nr] = elec_tmp
            else:
                print "Error: Could not read in energies from calculations."
                print "Please make sure that sander worked properly."
                return
            sys.stdout.write("\r") 
        test = file("probes.txt", "w")
        for k, v in probe_results.iteritems():
            test.write(str(k) + " " + str(v) + "\n")
            test.write("\n")
        test.close()
        surf_bfact = SinglePointAnalyzes.writeElecDiffAsBfac(self.pdb, surf, probe_results, elec)  
