'''
Created on 17.11.2010

@author: sschneider
'''

import sys, os, div
import Data_Handling
from Analyzes import SinglePointAnalyzes
from IO import PDBReader, PDBWriter
from Structure import  Protein


class AmberGB:
    
    def __init__(self, pdb, probe, cs1, distFac):
        self.pdb = pdb
        self.probe = probe
        self.cs1 = cs1
        self.distFac = distFac        
        
    """
    creates a new pdb file with the initial probe iplaced in the origin
    """
    def initPDBforGB(self):
        p = open(self.pdb + ".pdb")
        out = file(self.pdb + "_new.pdb", "w")
        at_nr = 0
        chain = 0        
        lines = p.readlines()
        for line in lines:
            if line.find("END") != -1:
                continue
            out.write(line)           
            try:
                ele = line.split()
                at_nr = int(ele[1])
                chain = int(ele[4])
            except:
                pass
        last_line = lines[len(lines) - 1]
        tmp_ele = last_line.split()        
        if len(ele) == 0 or tmp_ele[0] != "TER":
            out.write("TER \n")
        out.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", at_nr + 1, "M", "G", " ", "", "MG2",
                                        "", chain + 1, "", 0.00, 0.00, 0.00, 1.00, 0.00, "",
                                        "", "")))
        out.write("\n")
        out.write("TER \n")
        out.write("END \n")
        return self.pdb + "_new"
    
    """
    Writes the Sander input file
    """
    def writePBSAgb(self, offset):
        f = file("pbsa_gb.in", "w")
        f.write("\n")
        f.write("File generated by mm_pbsa.pl. Using AmberGB\n")
        f.write(" &cntrl \n")
        f.write("  ntf=1,   ntb=0, \n")
        f.write("  igb=5,   dielc=1.0, surften=0.0072\n")
        f.write("  cut=60.0, \n")
        f.write("  imin=1,  gbsa=2,  maxcyc=0, ntmin=2, \n")
        f.write("  rgbmax=20.00, saltcon=0.1 \n")
        f.write("  temp0=300.00, extdiel=80.0, intdiel=4.0,\n")
        f.write("&end \n")
        f.close()
    
    """
    Method to change the position of the initial probe
    to the desired position.    
    """
    def editCRD(self, pdb, pointCoordinates):
        crd = open(pdb + ".crd")
        lines = crd.readlines()
        line2edit = lines[len(lines) - 1]
        ele = line2edit.split()
        if len(ele) == 3:
            ele[0] = pointCoordinates.x
            ele[1] = pointCoordinates.y
            ele[2] = pointCoordinates.z
        elif len(ele) == 6:
            ele[3] = pointCoordinates.x
            ele[4] = pointCoordinates.y
            ele[5] = pointCoordinates.z
        elif len(ele) < 3:
            try:
                ele[0] = pointCoordinates.x
            except:
                ele.append(pointCoordinates.x)
            try:
                ele[1] = pointCoordinates.y
            except:
                ele.append(pointCoordinates.y)
            try:
                ele[2] = pointCoordinates.z
            except:
                ele.append(pointCoordinates.z)
        elif len(ele) < 6:
            while len(ele) > 0:
                del ele[0]
            ele.append(float(line2edit[0:12]))
            ele.append(float(line2edit[12:25]))
            ele.append(float(line2edit[25:36]))
            ele.append(pointCoordinates.x)
            ele.append(pointCoordinates.y)
            ele.append(pointCoordinates.z)
                
        else:            
            print "ERROR: CRD file corrupted: " + self.pdb + ".crd"            
            sys.exit()
        crd.close()
        crd = file(self.pdb + "_new.crd", "w")
        for i in range(len(lines)):
            if i == len(lines) - 1:
                for e in ele:                    
                    crd.write(str("%12.7f" % (float(e))))                    
                crd.write("\n")
            else:
                crd.write(lines[i])
    
    """
    Method to change the AMBER topology file initially.
    Charge is set to 0.0 and size is set to the given size 
    """        
    def editTOP(self, pdb, probe):
        top = open(pdb + ".top")        
        lines = top.readlines()
        for i in range(len(lines)):
            line = lines[i]
            ele = line.split()
            if len(ele) > 1 and ele[1] == "MASS":
                tmp_line = lines[i - 1]
                tmp_ele = tmp_line.split()
                tmp_ele[len(tmp_ele) - 1] = str("0.00000000E+00")
                tmp_str = ""
                for e in tmp_ele:
                    if float(e) >= 0.0:
                        tmp_str += "  " + e
                    else:
                        tmp_str += " " + e
                tmp_str += "\n"
                lines[i - 1] = tmp_str
            if len(ele) > 1 and ele[1] == "SCREEN":
                tmp_line = lines[i - 1]
                tmp_ele = tmp_line.split()
                tmp_ele[len(tmp_ele) - 1] = str(probe) + str("0000000E+00")
                tmp_str = ""
                for e in tmp_ele:
                    if float(e) > 0.0:
                        tmp_str += "  " + e
                    else:
                        tmp_str += " " + e
                tmp_str += "\n"
                lines[i - 1] = tmp_str
        out = file(self.pdb + "_new.top", "w")
        for line in lines:
            out.write(line)
            
    def amberGB(self, sp, bp, reuse, test):        

        Data_Handling.createSolvDatNew()                

        acceptable = {"ALA":0, "ARG":0, "ASN":0, "ASP":0, "CYS":0, "GLU":0, "GLN":0, "GLY":0, "HIS":0, "ILE":0, "LEU":0, "LYS":0, "MET":0, "PHE":0, "PRO":0, "SER":0, "THR":0, "TRP":0, "TYR":0, "VAL":0}
        """
        TEST correction
        """
        read = PDBReader.PDBReader()
        cor_prot = read.readPDB(self.pdb + ".pdb", False)
        new_cor_prot = []
        for a in cor_prot.atoms:
            if a.vdw > 0.0 and a.res_name in acceptable:
                new_cor_prot.append(a.copy())  
        Data_Handling.createTleapForAmberGB(self.pdb, True)           
        wr = PDBWriter.PDBWriter()
        wr.writePDB(self.pdb + ".pdb", Protein.Protein(new_cor_prot), False)
        
        self.writePBSAgb(0.09)
        try:
            os.system("rm -f leap.log")    
            os.system("tleap -f " + self.pdb + ".tleap > muell.txt")                        
            os.system("sander -O -i pbsa_gb.in -o gb_ref.out -c ./" + self.pdb + ".crd -p ./" + self.pdb + ".top > muell.txt")
            ref_elec = Data_Handling.getElecFromGB("gb_ref.out") * 4.18
        except:
            print "Error: Could not read in energies from calculations."
            print "Please make sure that sander worked properly."
            return


        tmp_protCalc = read.readPDB(self.pdb + ".pdb")
        print "calc probes"
        self.surfpoints = Protein.Protein(div.calcSmallProbes(tmp_protCalc, self.probe))
        countSP = 1
        for s in self.surfpoints.atoms:
            s.at_nr = countSP
            countSP += 1
        wr.writePDB(self.pdb + ".spt", self.surfpoints)
        nrs_surfpoints = len(self.surfpoints.atoms)
        
        
        if test == True:        
            return
        newpdb = self.initPDBforGB()        
        Data_Handling.createTleapForAmberGB(newpdb, False)
        os.system("rm -f leap.log")    
        os.system("tleap  -f " + newpdb + ".tleap > muell.txt")        
        self.editTOP(newpdb, self.probe)
        probe_results = {}
        print "***********************************"
        print
        count = 1
        for s in self.surfpoints.atoms:
            print "calc " + str(count) + " out of " + str(nrs_surfpoints),
            sys.stdout.flush()
            count += 1
            self.editCRD(newpdb, s)        
            os.system("sander -O -i pbsa_gb.in -o gb.out -c ./" + newpdb + ".crd -p ./" + newpdb + ".top > muell.txt")
            elele = Data_Handling.getElecFromGB("gb.out")
            if elele != None:            
                probe_results[s.at_nr] = elele * 4.18
            else:
                print "EXIT STAGE 1 by Sebastian"
                return
            sys.stdout.write("\r")
        test = file("probe.txt", "w")
        for k, v in probe_results.iteritems():
            test.write(str(k) + " " + str(v))
            test.write("\n")
        test.close()        
        surf_bfact = SinglePointAnalyzes.writeElecDiffAsBfac(self.pdb, self.surfpoints, probe_results, ref_elec)
        
