'''
Created on 17.11.2010

@author: sschneider
'''

import sys, os
import Data_Handling
from Analyzes import SinglePointAnalyzes
from SurfPointCreator import SinglePointsInCavityGenerator
from IO import PDBReader, PDBWriter
from Structure import  Protein
from Prediction import PredictionMethods


class AmberPB:
    
    def __init__(self, pdb, probe, cs1, distFac):
        self.pdb = pdb
        self.probe = probe
        self.cs1 = cs1
        self.distFac = distFac        
 
    """
    creates a new pdb file with the initial probe iplaced in the origin
    """
    def initPDBforPB(self):
        p = open(self.pdb + ".pdb")
        out = file(self.pdb + "_new.pdb", "w")
        at_nr = 0
        chain = 0        
        lines = p.readlines()
        for line in lines:
            if line.find("END") != -1:
                continue
            out.write(line)           
            try:
                ele = line.split()
                at_nr = int(ele[1])
                chain = int(ele[4])
            except:
                pass
        last_line = lines[len(lines) - 1]
        tmp_ele = last_line.split()        
        out.write(str("%-6s%5i %2s%1s%1s%1s%3s %1s%4i%1s   %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%2s" 
                                      % ("ATOM", at_nr + 1, "M", "G", " ", "", "MG2",
                                        "", chain + 1, "", 0.00, 0.00, 0.00, 1.00, 0.00, "",
                                        "", "")))
        out.write("\n")
        out.write("TER \n")
        out.write("END \n")
        return self.pdb + "_new"       
        
    """
    Old test method
    """
    def writePBSApbOld(self, epsout):
        f = file("pbsa.in", "w")
        f.write("File generated by mm_pbsa.pl. Using PB \n")
        f.write(" &cntrl \n")
        f.write("  ntf    = 1,       ntb    = 0, \n")
        f.write("  igb    = 10,      dielc  = 1.0, \n")
        f.write("  cut    = 999.0,   nsnb   = 99999, \n")
#        f.write("  scnb   = 2.0,     scee   = 1.2, \n")
        f.write("  imin   = 1,       maxcyc = 0,       ntmin  = 2, \n")
        f.write("  temp0 = 298.15, \n")
        f.write(" &end \n")
        f.write(" &pb \n")
        f.write("  epsin  = 10.0,     epsout  = " + str(epsout) + ", \n")
        f.write("  istrng = 0,       radiopt = 0, \n")
        f.write("  sprob  = 1.4,     space = 0.3, \n")
        f.write("  nbuffer = 0, fillratio = 2.0, \n")
        f.write("  maxitn = 1000,    npopt   = 1, \n")
        f.write("  cavity_surften = 0.0378,    cavity_offset   = 0.5692, \n")
        f.write("  npbverb= 1, \n")
        f.write(" &end \n")
        f.close()
    
    """
    Writes the Sander input file
    """    
    def writePBSApb(self, epsout):
        f = file("pbsa.in", "w")
        f.write("File generated by mm_pbsa.pl. Using PB \n")
        f.write(" &cntrl \n")
        f.write("  ntx    = 1, igb=10, ipb=1, inp=2, ntb=0 irest= 0, \n")
        f.write("  ntc=1, ntf=1 \n")        
        f.write("  cut    = 100.0, \n")
        f.write("  imin   = 1, maxcyc = 0, ntmin  = 2, \n")
        f.write("  temp0 = 300.00, \n")
        f.write(" &end \n")
        f.write(" &pb \n")
        f.write("  npbverb=1, epsin=4.0, epsout=80.0, \n")
        f.write("  radiopt = 1, \n")
        f.write("  space = 0.5, \n")
        f.write("  nbuffer = 0, fillratio = 4.0, accept=0.001, \n")
        f.write("  dbfopt=1, \n")
#        f.write("  cavity_surften = 0.0378,    cavity_offset   = 0.5692, \n")        
        f.write(" &end \n")
        f.close()
    
    
    """
    Method to change the position of the initial probe
    to the desired position.    
    """
    def editCRD(self, pdb, pointCoordinates):
        crd = open(pdb + ".crd")
        lines = crd.readlines()
        line2edit = lines[len(lines) - 1]
        ele = line2edit.split()
        if len(ele) == 3:
            ele[0] = pointCoordinates.x
            ele[1] = pointCoordinates.y
            ele[2] = pointCoordinates.z
        elif len(ele) == 6:
            ele[3] = pointCoordinates.x
            ele[4] = pointCoordinates.y
            ele[5] = pointCoordinates.z
        elif len(ele) < 3:
            try:
                ele[0] = pointCoordinates.x
            except:
                ele.append(pointCoordinates.x)
            try:
                ele[1] = pointCoordinates.y
            except:
                ele.append(pointCoordinates.y)
            try:
                ele[2] = pointCoordinates.z
            except:
                ele.append(pointCoordinates.z)
        elif len(ele) < 6:
            while len(ele) > 0:
                del ele[0]
            ele.append(float(line2edit[0:12]))
            ele.append(float(line2edit[12:25]))
            ele.append(float(line2edit[25:36]))
            ele.append(pointCoordinates.x)
            ele.append(pointCoordinates.y)
            ele.append(pointCoordinates.z)
                
        else:            
            print "ERROR: CRD file corrupted: " + self.pdb + ".crd"            
            sys.exit()
        crd.close()
        crd = file(self.pdb + "_new.crd", "w")
        for i in range(len(lines)):
            if i == len(lines) - 1:
                for e in ele:                    
                    crd.write(str("%12.7f" % (float(e))))                    
                crd.write("\n")
            else:
                crd.write(lines[i])
    """
    Method to change the AMBER topology file initially.
    Charge is set to 0.0 and size is set to the given size 
    """            
    def editTOP(self, pdb, probe):
        top = open(pdb + ".top")        
        lines = top.readlines()
        for i in range(len(lines)):
            line = lines[i]
            ele = line.split()
            if len(ele) > 1 and ele[1] == "MASS":
                tmp_line = lines[i - 1]
                tmp_ele = tmp_line.split()
                tmp_ele[len(tmp_ele) - 1] = str("0.00000000E+00")
                tmp_str = ""
                for e in tmp_ele:
                    if float(e) >= 0.0:
                        tmp_str += "  " + e
                    else:
                        tmp_str += " " + e
                tmp_str += "\n"
                lines[i - 1] = tmp_str
            if len(ele) > 1 and ele[1] == "SCREEN":
                tmp_line = lines[i - 1]
                tmp_ele = tmp_line.split()
                tmp_ele[len(tmp_ele) - 1] = str(probe) + str("0000000E+00")
                tmp_str = ""
                for e in tmp_ele:
                    if float(e) > 0.0:
                        tmp_str += "  " + e
                    else:
                        tmp_str += " " + e
                tmp_str += "\n"
                lines[i - 1] = tmp_str
        out = file(self.pdb + "_new.top", "w")
        for line in lines:
            out.write(line)
            
    def amberPB(self, sp, bp, reuse, test):
        read = PDBReader.PDBReader()
        print self.pdb + ".pdb", os.getcwd()
        cor_prot = read.readPDB(self.pdb + ".pdb", True)
        new_cor_prot = []
        for a in cor_prot.atoms:
            if a.vdw > 0.0:
                new_cor_prot.append(a.copy()) 
            
        wr = PDBWriter.PDBWriter()
        wr.writePDB(self.pdb + ".pdb", Protein.Protein(new_cor_prot), False)
        
        Data_Handling.createSolvDat()        
        Data_Handling.createTleapForAmberPB(self.pdb, True)                
        os.system("rm -f leap.log")    
        os.system("tleap -f " + self.pdb + ".tleap")
        self.writePBSApb(78.54)
        os.system("sander -O -i pbsa.in -o pb_ref.out -c ./" + self.pdb + ".crd -p ./" + self.pdb + ".top")            
        ref_elec = Data_Handling.getElecFromPB("pb_ref.out") 
        pdb = self.initPDBforPB()
        sic = SinglePointsInCavityGenerator.ProbesInCavity(self.pdb)
        nrs_surfpoints, self.surfpoints = sic.generateProbesInCavity(sp, bp, self.cs1, self.distFac, reuse)
        Data_Handling.createTleapForAmberPB(pdb, False)
        os.system("rm -f leap.log")    
        os.system("tleap -s -f leaprc.ff99SB -f " + pdb + ".tleap")        
        self.editTOP(pdb, self.probe)
        probe_results = {}

        count = 1
        for s in self.surfpoints.atoms:
            print "calc " + str(count) + " out of " + str(nrs_surfpoints),
            sys.stdout.flush()
            count += 1
            self.editCRD(pdb, s)                            
            os.system("sander -O -i pbsa.in -o pb.out -c ./" + pdb + ".crd -p ./" + pdb + ".top")            
            elele = Data_Handling.getElecFromPB("pb.out")
            if elele != None:            
                probe_results[s.at_nr] = elele 
            else:
                print "Error: Could not read in energies from calculations."
                print "Please make sure that sander worked properly."
                return
            sys.stdout.write("\r")
        test = file("probes.txt", "w")
        for k, v in probe_results.iteritems():
            test.write(str(k) + " " + str(v) + "\n")
            test.write("\n")
        test.close()
        surf_bfact = SinglePointAnalyzes.writeElecDiffAsBfac(self.pdb, self.surfpoints, probe_results, ref_elec)
        predicted = PredictionMethods.predictCavities(surf_bfact, self.cs1 * 0.84, self.distFac)
        wr.writeSP(self.pdb + "_prediction.pdb", predicted.atoms)
