'''
Created on 24.04.2012

@author: sschneider
'''


from Analyzes import AnalyzeMethods 
from Structure import  Protein
from SurfPointCreator import CavityProbeIdentifier
import Cavity

def getRelevantCavities(cavities, nrLigands):    
    largestClusters = {}    
    while len(largestClusters) < nrLigands:        
        larg = -1
        tmp = None
        n = None
        for nr, cav in cavities.iteritems():
            if nr in largestClusters:
                continue
            if len(cav.points) > larg:
                larg = len(cav.points)
                tmp = cav
                n = nr
        if tmp == None:
            return largestClusters
        largestClusters[n] = tmp    
    return largestClusters

def generateCavities(prediction):
    cavs = {}
    for p in prediction.atoms:
        if p.res_nr not in cavs:
            cavs[p.res_nr] = Cavity.Cavity()
        cavs[p.res_nr].points.append(p.copy())
    return cavs



def reorderResiduesInProtein(protein, startingNumber=1):
    old_res_nr = 0
    temp_res_nr = 0
    for atom in protein.atoms:
        if old_res_nr == 0:
            temp_res_nr = startingNumber
            old_res_nr = atom.res_nr
        if atom.res_nr == old_res_nr:
            atom.res_nr = temp_res_nr
        else:
            temp_res_nr += 1
            old_res_nr = atom.res_nr
            atom.res_nr = temp_res_nr                
    return protein

def getdeltaGValuesForCavities(cavs):
    min = 1000.0
    max = -1.0
    ave = 0.0
    count = 0.0
    for nr, cav in cavs.iteritems():
        for a in cav.points:
            if a.temp > max:
                max = a.temp
            if a.temp < min:
                min = a.temp
            ave += a.temp
            count += 1.0
    try:
        ave /= count
    except:
        ave = 9999
    return max, min, ave    


def returnByValueRange(prediction, low, high):
    ret = []
    for a in prediction.atoms:
        if a.temp > low and a.temp < high:
            ret.append(a)
    return ret

def clusterByValue(prediction, dist):
    cavs = {}
    """
    generate cavities
    """
    min = 100.0
    for a in prediction.atoms:
        a.occ = 0.0
        if a.temp < min:
            min = a.temp
        if a.res_nr not in cavs:
            cavs[a.res_nr] = []
        cavs[a.res_nr].append(a.copy())                
    """
    calculate clusters per cavity
    """
    allCavs = []     
    for nr, cav in cavs.iteritems():
        rtCavs = {}
        for a in cav:            
            start = min
            while a.temp > start:                
                start += 0.6
            if start not in rtCavs:
                rtCavs[start] = []
            rtCavs[start].append(a.copy())
            a.occ = start
        allCavs.append(rtCavs)
    """
    """
    count = 1
    ret = Protein.Protein([])
    for rtCav in allCavs:
        for rt, atoms in rtCav.iteritems():
            tmp = CavityProbeIdentifier.filterProbesOutsideCavity(Protein.Protein(atoms), 1, dist)
            if len(tmp) == 0:
                continue
            tmp = reorderResiduesInProtein(Protein.Protein(tmp), count).atoms
            count = tmp[-1].res_nr + 1
            ave = 0.0
            for a in tmp:
                ave += a.temp
            ave /= float(len(tmp))
            for a in tmp:
                a.temp = ave
                if a.temp > abs(5.0 * min):
                    a.chain_id = "P"
                elif a.temp > abs(3.0 * min):
                    a.chain_id = "M"
                else:
                    a.chain_id = "N"
                ret.atoms.append(a.copy())           
    return ret
            

            
def singleCavityDecision(predPoints, dist, clSize):
    prediction = AnalyzeMethods.deleteContactAreas(Protein.Protein(predPoints), 1.4, 7)    
    prediction = AnalyzeMethods.deleteContactAreas(prediction, 1.4, 7)
    return prediction

def lowNrOfCavitiesDecision(predPoints, dist, clSize):        
    try:        
        highest = AnalyzeMethods.takeHighestFromBfact(Protein.Protein(predPoints).copy(), 0.4)
        highest = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(highest, int(clSize / 3.0), dist))        
        lowest = AnalyzeMethods.takeLowestFromBfact(Protein.Protein(predPoints).copy(), clSize, dist, 0.6)
    except:
        print "Fatal : could not extract highest and lowest desolvation energies",
        return Protein.Protein([])           
    for a in highest.atoms:
        lowest.atoms.append(a)
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(lowest, clSize, dist))
    prediction = AnalyzeMethods.deleteContactAreas(prediction, 1.4, 7)  
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(prediction, clSize, dist))  
    return prediction


def highNrOfCavitiesDecision(predPoints, dist, clSize):
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(Protein.Protein(predPoints), clSize, dist))
    try:        
        highest = AnalyzeMethods.takeHighestFromBfact(prediction.copy(), 0.3) 
        highest = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(highest, int(clSize / 2.0), dist))        
        lowest = AnalyzeMethods.takeLowestFromBfact(prediction.copy(), clSize, dist, 0.7)
    except:
        print "Fatal : could not extract highest and lowest desolvation energies",
        return Protein.Protein([])     
    for a in highest.atoms:
        lowest.atoms.append(a)
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(lowest, clSize, dist))
    prediction = AnalyzeMethods.deleteContactAreas(prediction, 1.4, 7)  
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity(prediction, clSize, dist))  
    return prediction

def predictCavities(bfact, minClSize, distFac):
    print """
    
Starting prediction...
"""    
    cavs = generateCavities(bfact)
    cluster = getRelevantCavities(cavs, 50000)
    predPoints = []        
    for nr, cav in cluster.iteritems():
        for p in cav.points:
            predPoints.append(p.copy())    
    print "initial nr of points:", len(predPoints)
    if len(cavs) == 1:
        prediction = singleCavityDecision(predPoints, distFac, minClSize)
        print "... only one cavity found ... "
    else:                    
        if len(cavs) <= 3:
            prediction = lowNrOfCavitiesDecision(predPoints, distFac, minClSize)
            print "... ",len(cavs), "cavities found ..."
        else:
            prediction = highNrOfCavitiesDecision(predPoints, distFac, minClSize)
            print "... ",len(cavs), "cavities found ..."
    prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity((prediction), minClSize, distFac))    
    if len(prediction.atoms) < minClSize:
        print "no cluster with the minimum number of probes found"
        prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity((Protein.Protein(predPoints)), minClSize, distFac))
        if len(prediction.atoms) < minClSize:
            print "... returning all probes clustered if possible"
            prediction = Protein.Protein(predPoints)
            prediction = Protein.Protein(CavityProbeIdentifier.filterProbesOutsideCavity((Protein.Protein(predPoints)), 2, distFac))
        else:
            print "... clustered all initial probes to clusters of minimum size. No energy differences included in prediction."
    print "final nr of probes:", len(prediction.atoms),    
    return prediction
    
