'''
Created on 03.06.2011

@author: sschneider
'''
import copy

#===============================================================================
# CLASS ATOM
#===============================================================================
class Atom:    
    
    
    """
    The atom class represents atoms like they were described in the PDB file format. 
    (see U{http://www.pdb.org} for more informations)
    
    
    @ivar at_nr: the atom number        
    @type at_nr: int    
    @ivar at_name: the name of the atom
    @type at_name: string    
    @ivar rem_indic: Distance abbreviation
    @type rem_indic: string
    @ivar branch_des: digit designation to branch direction
    @type branch_des: string
    @ivar alt_loc: alternative position indicator
    @type alt_loc: string    
    @ivar res_name: the name of the residue the atom belongs to
    @type res_name: string    
    @ivar chain_id: the chain the atom (resp. the residue) belongs to
    @type chain_id: string    
    @ivar res_nr: the number of the residue
    @type res_nr: int
    @ivar insertion_code: addition to residue number 
    @type insertion_code: string        
    @ivar x: the x position of the atom
    @type x: float    
    @ivar y: the y position of the atom
    @type y: float    
    @ivar z: the z position of the atom
    @type z: float    
    @ivar occ: atom occupancy
    @type occ: float    
    @ivar temp: B value or temperature factor of the atom
    @type temp: float    
    @ivar segID: segment identifier
    @type segID: string    
    @ivar element: element symbol
    @type element: string    
    @ivar charge: charge on the atom
    @type charge: string     
            
    """
                
    def __init__(self, at_nr, at_name, rem_indic, branch_des, alt_loc, res_name, chain_id, res_nr, insertion_code, x, y, z, occ, temp, segID, element, charge):
        """            
        This method initializes the atom with all necessary parameters
        
        @param at_nr: the atom number        
        @type at_nr: int    
        @param at_name: the name of the atom
        @type at_name: string    
        @param rem_indic: Distance abbreviation
        @type rem_indic: string
        @param branch_des: digit designation to branch direction
        @type branch_des: string
        @param alt_loc: alternative position indicator
        @type alt_loc: string    
        @param res_name: the name of the residue the atom belongs to
        @type res_name: string    
        @param chain_id: the chain the atom (resp. the residue) belongs to
        @type chain_id: string    
        @param res_nr: the number of the residue
        @type res_nr: int
        @param insertion_code: addition to residue number 
        @type insertion_code: string        
        @param x: the x position of the atom
        @type x: float    
        @param y: the y position of the atom
        @type y: float    
        @param z: the z position of the atom
        @type z: float    
        @param occ: atom occupancy
        @type occ: float    
        @param temp: B value or temperature factor of the atom
        @type temp: float    
        @param segID: segment identifier
        @type segID: string    
        @param element: element symbol
        @type element: string    
        @param charge: charge on the atom
        @type charge: string            
        """
        self.at_nr = at_nr
        self.at_name = at_name
        self.rem_indic = rem_indic
        self.branch_des = branch_des
        self.alt_loc = alt_loc
        self.res_name = res_name
        self.chain_id = chain_id
        self.res_nr = res_nr
        self.insertion_code = insertion_code
        self.x = x 
        self.y = y 
        self.z = z 
        self.occ = occ
        self.temp = temp        
        self.segID = segID
        self.element = element
        self.charge = charge
        self.vdw = 0.0
        if at_name == "H":
            self.vdw = 0.0
        if at_name == "C":
            self.vdw = 1.9
        if at_name == "N":
            self.vdw = 1.6
        if at_name == "O":
            self.vdw = 1.5
        if at_name == "P":
            self.vdw = 2.0
        if at_name == "S":
            self.vdw = 2.0        
    
    def copy(self):
        return Atom(copy.copy(self.at_nr), copy.copy(self.at_name), copy.copy(self.rem_indic), copy.copy(self.branch_des), copy.copy(self.alt_loc), copy.copy(self.res_name), copy.copy(self.chain_id), copy.copy(self.res_nr), copy.copy(self.insertion_code), copy.copy(self.x), copy.copy(self.y), copy.copy(self.z), copy.copy(self.occ), copy.copy(self.temp), copy.copy(self.segID), copy.copy(self.element), copy.copy(self.charge))    

    def printAtom(self):
        print str(self.at_nr) + " " + str(self.at_name) + " " + str(self.res_name) + " " + str(self.res_nr) + " " + str(self.chain_id) + " " + str(self.x) + " " + str(self.y) + " " + str(self.z) + " " + str(self.vdw) + " " + str(self.temp)  

    def distToOtherNoSqrt(self, other):
        return (self.x - other.x) * (self.x - other.x) + (self.y - other.y) * (self.y - other.y) + (self.z - other.z) * (self.z - other.z)
