'''
Created on 03.06.2011

@author: sschneider
'''
class SimpleAtom:
    """
    The SimpleAtom class represents atoms like Atom but can be initialized with only atom number, name and coordinates. 
    The rest is set default. 
    (For more Informations on PDB see U{http://www.pdb.org})
    
    @ivar at_nr: the atom number        
    @type at_nr: int    
    @ivar at_name: the name of the atom
    @type at_name: string    
    @ivar rem_indic: Distance abbreviation
    @type rem_indic: string
    @ivar branch_des: digit designation to branch direction
    @type branch_des: string
    @ivar alt_loc: alternative position indicator
    @type alt_loc: string    
    @ivar res_name: the name of the residue the atom belongs to
    @type res_name: string    
    @ivar chain_id: the chain the atom (resp. the residue) belongs to
    @type chain_id: string    
    @ivar res_nr: the number of the residue
    @type res_nr: int
    @ivar insertion_code: addition to residue number 
    @type insertion_code: string        
    @ivar x: the x position of the atom
    @type x: float    
    @ivar y: the y position of the atom
    @type y: float    
    @ivar z: the z position of the atom
    @type z: float    
    @ivar occ: atom occupancy
    @type occ: float    
    @ivar temp: B value or temperature factor of the atom
    @type temp: float    
    @ivar segID: segment identifier
    @type segID: string    
    @ivar element: element symbol
    @type element: string    
    @ivar charge: charge on the atom
    @type charge: string       
    """   
    
    def __init__(self, at_nr, at_name, x, y, z):
        """            
        This method initializes the atom with only number, name and coordinates.
        
        @param at_nr: the atom number        
        @type at_nr: int    
        @param at_name: the name of the atom
        @type at_name: string                        
        @param x: the x position of the atom
        @type x: float    
        @param y: the y position of the atom
        @type y: float    
        @param z: the z position of the atom
        @type z: float                        
        """
        self.at_nr = at_nr
        self.at_name = at_name
        self.rem_indic = " "
        self.branch_des = " "
        self.alt_loc = " "
        self.res_name = "XXX"
        self.chain_id = " "
        self.res_nr = 0
        self.insertion_code = " "
        self.x = x 
        self.y = y 
        self.z = z 
        self.occ = 0.00
        self.temp = 0.00        
        self.segID = " "
        self.element = " "
        self.charge = " "