'''
Created on 24.11.2010

@author: sschneider
'''
#from Molecules import PDB_IO, Protein
import copy

def distance(a, b):
    return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z)

class cavity:
    
    probes = {}
    
    def __init__(self, nr):
        self.nr = nr
    
    def isIn(self, probe):
        if probe.id in self.probes:
            return True
        elif probe.id not in self.probes:
            return False 
    
    def copy(self):
        new_probes = {}
        new_number = copy.copy(self.nr)
        for k, v in self.probes.iteritems():
            new_probes[copy.copy(k)] = v.copy()
        new_cav = cavity(new_number)
        new_cav.probes = new_probes
        return new_cav
    
    def delMe(self):
        self.probes.clear()
        self.nr = -1
        
    def printMe(self):        
        l = self.probes.keys()
        print "Cavity nr " + str(self.nr) + " has atoms nr: "
        print " ".join(l)
        print "as probes"


neigs = {}
def getNeighbours(all, atoms, dist):
    ret = []
    for a in all:
        for k, b in atoms.iteritems():
            if k in neigs:
                continue            
            if distance(a, b) <= dist:
                ret.append(b.copy())       
    return ret

"""
WARNING: RECURSION!!!
"""
def getAllNeighbours(aneig, atoms, dist):    
    if len(aneig) > 0:        
        res = []                
        tmpNeighbours = getNeighbours(aneig, atoms, dist)  
#        print len(tmpNeighbours)          
        for neighbour in tmpNeighbours:            
            if neighbour.id not in neigs:                
                neigs[neighbour.id] = neighbour.copy()
                res.append(neighbour.copy())     
        aneig = getAllNeighbours(res, atoms, dist)
        
def cluster(sps, nr_of_probes, dist_to_neig, sort=False, nrs=3, ave=False):    
    cavities = {}
    cav_count = 1
    for i in range(len(sps)):
        sps[i].id = i + 1
    probes = {}
    for a in sps:
        probes[a.id] = a.copy()
    while len(probes) > 0:
        keys = probes.keys()
        p = probes[keys[0]]
        tmp_cav = cavity(cav_count)    
        tmp_list = []
        tmp_list.append(p)
        neigs[p.id] = p.copy()    
        getAllNeighbours(tmp_list, probes, dist_to_neig)
        for k, v in neigs.iteritems():
            if tmp_cav.isIn(v) == False:
                tmp_cav.probes[v.id] = v.copy()
        neigs.clear()
        for k, v in tmp_cav.probes.iteritems():
            del probes[k]
        cavities[cav_count] = tmp_cav.copy()
        tmp_cav.delMe()        
        cav_count += 1
        while len(tmp_list) > 0:
            del tmp_list[0]
    leftOver = []
    if sort == False:   
        if ave == True:                     
            for cav_nr, cav in cavities.iteritems():                   
                aveTmp = 0.0
                for id_kk, probe in cav.probes.iteritems():                    
                    aveTmp = probe.temp
                aveTmp /= float(len(cav.probes))
                for id_kk, probe in cav.probes.iteritems():
                    probe.temp = aveTmp
        count = 0
        while count < nrs:
            largest = -1
            larg = 0
            for cav_nr, cav in cavities.iteritems():       
                if len(cav.probes) < nr_of_probes:
                    continue
                if len(cav.probes) > larg:
                    larg = len(cav.probes)
                    largest = cav_nr            
            if largest != -1:
                count += 1
                for id_kk, probe in cavities[largest].probes.iteritems():               
                    leftOver.append(probe.copy())
                del cavities[largest]
            else:
                return leftOver
            if len(cavities) == 0:
                return leftOver
    else:        
        count = 0        
        while count < nrs:            
            lowest = -1
            low = 1000.0
            if ave == True:
                for cav_nr, cav in cavities.iteritems():       
                    if len(cav.probes) < nr_of_probes:
                        continue
                    aveTmp = 0.0
                    for id_kk, probe in cav.probes.iteritems():
                        aveTmp = probe.temp
                    aveTmp /= float(len(cav.probes))
                    for id_kk, probe in cav.probes.iteritems():
                        probe.temp = aveTmp
                    
            for cav_nr, cav in cavities.iteritems():       
                if len(cav.probes) < nr_of_probes:
                    continue
                k = cav.probes.keys()
                for p,pp in cav.probes.iteritems():
                    if pp.temp < low:                                    
                        low = pp.temp
                        lowest = cav_nr
            count += 1       
            if lowest != -1:
                for id_kk, probe in cavities[lowest].probes.iteritems():
#                    probe.res_nr = lowest
                    probe.temp = low             
                    leftOver.append(probe.copy())
                del cavities[lowest]
            else:
                return leftOver
            if len(cavities) == 0:
                return leftOver
            """
            /end
            """

    return leftOver


def clusterForHigh(sps, nr_of_probes, dist_to_neig, sort=False, nrs=3):    
    cavities = {}
    cav_count = 1
    for i in range(len(sps)):
        sps[i].id = i + 1
    probes = {}
    for a in sps:
        probes[a.id] = a.copy()
    while len(probes) > 0:
        keys = probes.keys()
        p = probes[keys[0]]
        tmp_cav = cavity(cav_count)    
        tmp_list = []
        tmp_list.append(p)
        neigs[p.id] = p.copy()    
        getAllNeighbours(tmp_list, probes, dist_to_neig)
        for k, v in neigs.iteritems():
            if tmp_cav.isIn(v) == False:
                tmp_cav.probes[v.id] = v.copy()
        neigs.clear()
        for k, v in tmp_cav.probes.iteritems():
            del probes[k]
        cavities[cav_count] = tmp_cav.copy()
        tmp_cav.delMe()        
        cav_count += 1
        while len(tmp_list) > 0:
            del tmp_list[0]
    leftOver = []
    if sort == False:
        for cav_nr, cav in cavities.iteritems():
            if len(cav.probes) < nr_of_probes:
                continue
            for id_kk, probe in cav.probes.iteritems():
                probe.res_nr = cav_nr
                probe.temp = cav_nr             
                leftOver.append(probe)
    else:        
        count = 0        
        while count < nrs:
            largest = -1
            larg = 0
            highest = -1
            high = -1000.0
            for cav_nr, cav in cavities.iteritems():       
                if len(cav.probes) < nr_of_probes:
                    continue
                aveTmp = 0.0
                for id_kk, probe in cav.probes.iteritems():
                    aveTmp = probe.temp
                aveTmp /= float(len(cav.probes))
                for id_kk, probe in cav.probes.iteritems():
                    probe.temp = aveTmp
                    
            for cav_nr, cav in cavities.iteritems():       
                if len(cav.probes) < nr_of_probes:
                    continue
                k = cav.probes.keys()
                if cav.probes[k[0]].temp > high:                                    
                    high = cav.probes[k[0]].temp
                    highest = cav_nr
            count += 1       
            if highest != -1:
                for id_kk, probe in cavities[highest].probes.iteritems():
                    probe.res_nr = highest
                    probe.temp = high             
                    leftOver.append(probe.copy())
                del cavities[highest]
            else:
                return leftOver
            if len(cavities) == 0:
                return leftOver
            """
            /end
            """
#            for cav_nr, cav in cavities.iteritems():       
#                if len(cav.probes) < nr_of_probes:
#                    continue
#                if len(cav.probes) > larg:
#                    larg = len(cav.probes)
#                    largest = cav_nr
#            count += 1
#            if largest != -1:
#                for id_kk, probe in cavities[largest].probes.iteritems():
#                    probe.res_nr = larg
#                    probe.temp = larg             
#                    leftOver.append(probe.copy())
#                del cavities[largest]
#            else:
#                return leftOver
#            if len(cavities) == 0:
#                return leftOver
    return leftOver
