'''
Created on 16.11.2010

@author: sschneider
'''
import math
from Methods import Data_Handling
from IO import PDBReader, PDBWriter
from Structure import Point3D
import sys

class SinglePointGenerator:
    
    def __init__(self):
        pass
    
    def reuseAlreadyCalculatedSurfacePoints(self, pdb):
        return Data_Handling.readInSurfacePoints(pdb + ".spt")
    
    def generateSurfacePoints(self, probesize, pdb):
        reader = PDBReader.PDBReader()
        prot = reader.readPDB(pdb + ".pdb")
        out = self.calc(prot, probesize)
        writer = PDBWriter.PDBWriter()
        writer.writeSP(pdb + ".spt", out)
        return Data_Handling.readInSurfacePoints(pdb + ".spt")
    
    def distance(self, a, b):
        return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z)
    
    def distanceSQRT(self, a, b):
        return math.sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z))   
    
    def calc(self, pdb, srad):
        print "Collecting surface atoms, this may last several seconds..."
        nphi = 25;
        ncosth = 10;        
        natoms = len(pdb.atoms)        
        csth = []
        snth = []
        phgh = []
        cos_phigh = []
        sin_phigh = [] 
        neigh = []   
        sasa = []        
        for ik in range(ncosth):        
            csth.append(-1.0 + ((ik + 1) - 0.5) * (2.0 / (ncosth)))
            snth.append(math.sqrt(1.0 - csth[ik] * csth[ik]))    
        for ik in range(nphi):
            phgh.append(2.0 * math.pi * (ik) / (nphi));    
        for kk in range(nphi):
            cos_phigh.append(math.cos(phgh[kk]))
            sin_phigh.append(math.sin(phgh[kk]))
        
        for i in range(natoms):                          
            for jj in range(natoms):
                if jj == i:
                    continue;
                ccdist = math.sqrt(self.distance(pdb.atoms[i], pdb.atoms[jj]))
                rr = (pdb.atoms[i].vdw + pdb.atoms[jj].vdw + 2.0 * srad)            
                if (ccdist <= rr):
                    neigh.append(pdb.atoms[jj])    
            for jj in range(ncosth):
                costh = csth[jj];
                sinth = snth[jj];
                for kk in range(nphi):                
                    xp = (pdb.atoms[i].vdw + srad) * sinth * cos_phigh[kk];
                    yp = (pdb.atoms[i].vdw + srad) * sinth * sin_phigh[kk];
                    zp = (pdb.atoms[i].vdw + srad) * costh;
                    ll = 0;
                    coverd = False;
                    while (coverd == False and ll < len(neigh)):
                        if (pow((pdb.atoms[i].x + xp - neigh[ll].x), 2) + pow((pdb.atoms[i].y + yp
                                - neigh[ll].y), 2) + pow((pdb.atoms[i].z + zp - neigh[ll].z), 2)) < pow((neigh[ll].vdw + srad), 2):                        
                            coverd = True;                    
                        ll += 1;                
                    if (coverd == False):
                        sasa.append(Point3D.Point3D(pdb.atoms[i].x + xp, pdb.atoms[i].y + yp, pdb.atoms[i].z + zp))                                                 
            while len(neigh) > 0:
                del neigh[0]              
        for i in range(len(sasa)):                
            if sasa[i].nr == -1:
                continue
            print str(i) + "/" + str(len(sasa)),
            sys.stdout.flush() 
            for j in range(len(sasa)):                
                if sasa[j].nr == -1 or i == j:
                    continue
                if self.distance(sasa[i], sasa[j]) < 7.0:
                    sasa[j].nr = -1
            sys.stdout.write("\r")
        print
        print "finished"
        out = []
        for a in sasa:
            if a.nr != -1:
                out.append(a)
        for o in out:
            nearest = -1
            dist = 10000
            for p in pdb.atoms:
                if self.distance(o, p) < dist:
                    dist = self.distance(o, p)
                    nearest = p.res_nr
            o.res_nr = nearest       
        return out    