'''
Created on 16.11.2010

@author: sschneider
'''

from IO import PDBReader, PDBWriter
import math
import CavityProbeIdentifier
from Analyzes import AnalyzeMethods
from Methods import Data_Handling
from Structure import Protein, Atom



class ProbesInCavity:
    
    def __init__(self, pdb):        
        reader = PDBReader.PDBReader()
        self.name = pdb
        self.pdb = reader.readPDB(pdb + ".pdb", False)
    
    def calcSmallProbes(self, pdb, srad=1.4, fac=1.0):    
        print "--- settings"   
        nphi = 30;
        fac2 = fac * fac
        ncosth = 20;    
        print "     nphi", nphi
        print "     ncosth", ncosth
        print "     probesize", srad    
        natoms = len(pdb.atoms)        
        csth = []
        snth = []
        phgh = []
        cos_phigh = []
        sin_phigh = [] 
        neigh = []   
        sasa = []            
        for ik in range(ncosth):        
            csth.append(-1.0 + ((ik + 1) - 0.5) * (2.0 / (ncosth)))
            snth.append(math.sqrt(1.0 - csth[ik] * csth[ik]))    
        for ik in range(nphi):
            phgh.append(2.0 * math.pi * (ik) / (nphi));    
        for kk in range(nphi):
            cos_phigh.append(math.cos(phgh[kk]))
            sin_phigh.append(math.sin(phgh[kk]))
        
        for i in range(natoms):        
            neigh[:] = []                  
            for j in range(natoms):
                if j == i:
                    continue        
                if self.distance(pdb.atoms[i], pdb.atoms[j]) <= (pdb.atoms[i].vdw + pdb.atoms[j].vdw + 2.0 * srad):
                    neigh.append(pdb.atoms[j])    
            for jj in range(ncosth):
                costh = csth[jj];
                sinth = snth[jj];
                for kk in range(nphi):                
                    xp = (pdb.atoms[i].vdw + srad) * sinth * cos_phigh[kk];
                    yp = (pdb.atoms[i].vdw + srad) * sinth * sin_phigh[kk];
                    zp = (pdb.atoms[i].vdw + srad) * costh;
                    ll = 0;
                    coverd = False;
                    while (not coverd and ll < len(neigh)):
                        if (pow((pdb.atoms[i].x + xp - neigh[ll].x), 2) + pow((pdb.atoms[i].y + yp
                                - neigh[ll].y), 2) + pow((pdb.atoms[i].z + zp - neigh[ll].z), 2)) < pow((neigh[ll].vdw + srad), 2):                        
                            coverd = True;                    
                        ll += 1;                
                    if (coverd == False):
    #                    tmp_p = Point3D.Point3D(pdb.atoms[i].x + xp, pdb.atoms[i].y + yp, pdb.atoms[i].z + zp)
                        tmp_p = Atom.Atom(1, "H", "", "", "", "SPS", "", 1, "", pdb.atoms[i].x + xp, pdb.atoms[i].y + yp, pdb.atoms[i].z + zp, 0.0, 0.0, "", "", "")
                        tmp_bool = False
                        for a in sasa:
                            if self.distanceNoSQRT(a, tmp_p) < fac2:
                                tmp_bool = True
                                break
                        if tmp_bool == False:                                   
                            sasa.append(tmp_p.copy())                                                 
        return sasa

    def isIn(self, listOfProbes, probe):
        for a in listOfProbes:
            if a.at_nr == probe.at_nr:
                return True
        return False
    
    def distanceNoSQRT(self, a, b):
        return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z)
    
    def distance(self, a, b):
        return math.sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z))
    
    def generateProbesInCavity(self, smallSPrad, bigSPrad, cs1, distFac, reuse):
        #generate all probes using shrake algorithm
        writer = PDBWriter.PDBWriter()
        reader = PDBReader.PDBReader()        
        fac = distFac
        for a in self.pdb.atoms:
            a.temp = 0.0
        if reuse == False:            
            print "calc small probes"
            smallProbes = self.calcSmallProbes(self.pdb, smallSPrad, smallSPrad / 2.0)# - smallSPrad * 0.2)
            PDBWriter.PDBWriter().writePDB("small.points", Protein.Protein(smallProbes), False)
            print "calc big probes"
            largeProbes = self.calcSmallProbes(self.pdb, bigSPrad, 2.0)
            PDBWriter.PDBWriter().writePDB("large.points", Protein.Protein(largeProbes), False)
            nr = 1
            print "filter probes"
            for s in smallProbes:
                s.at_nr = nr
                nr += 1            
            new_smallProbes = []
            for s in smallProbes:
                hitted = False
                for b in largeProbes:
                    if self.distance(s, b) < smallSPrad + bigSPrad:
                        hitted = True
                        break
                if not hitted:
                    new_smallProbes.append(s.copy())
            print "nr of raw probes", len(new_smallProbes)
            PDBWriter.PDBWriter().writePDB("raw.points", Protein.Protein(new_smallProbes), False)
            sps = Protein.Protein(new_smallProbes)
        else:
            sps = reader.readPDB("raw.spt", False)
            
#            writer.writePDBForCppSP("used_prot.pdb", self.pdb)
#            os.system("~/projekte/sourcecode/C_Cpp/DesolvPrediction/SurfacePointShrake/Debug/SurfacePointShrake used_prot.pdb " + self.name + "_raw.spt " + str(smallSPrad) + " " + str(bigSPrad))        
#        sps = reader.readPDB(self.name + "_raw.spt", False)
#        sps = Protein.Protein(new_smallProbes)
        nearness = int(cs1 / 4.0)
        if nearness > 6:
            nearness = 6
                
        leftOver = (CavityProbeIdentifier.filterProbesOutsideCavity(sps.copy(), int(cs1), distFac))
        noNarrow = AnalyzeMethods.deleteContactAreas(Protein.Protein(leftOver), distFac, 5)        
        leftOver = CavityProbeIdentifier.filterProbesOutsideCavity(noNarrow, int(cs1), distFac)

        if len(leftOver) < cs1:
            allProbes = CavityProbeIdentifier.filterProbesOutsideCavity(sps.copy(), cs1, 2.0 * distFac)
            if len(allProbes) < cs1:
                allProbes = CavityProbeIdentifier.filterProbesOutsideCavity(sps.copy(), int(cs1 / 4.0), 2.0)
            writer.writeSP(self.name + ".spt", allProbes)
            print "Clustering left no probes at all, continue with all probes instead -->", len(allProbes)
        else:            
            writer.writeSP(self.name + ".spt", leftOver)
            print
        return Data_Handling.readInSurfacePoints(self.name + ".spt")

                
