'''
Created on 14.10.2011

@author: sschneider
'''
from Methods import SanderGB_Cavities, APBS_Cavities, SanderPB_Cavities, SanderGB
import os
from optparse import OptionParser
desc = """
*** dPred ***

A program to calculate the desolvation penalties of small probes 
in cavities to identify possible binding sites of small ligands.

To use desolvation penalty calculations, one of the following
prerequisites has to be fulfilled:

To use the Generalized Born approach, the AMBER software suite 
has to be installed properly (see http://ambermd.org/ for more 
informations).

To calculate the desolvation penalties using the 
Poisson-Boltzmann (PB) method, two possible approaches are 
supported (beta status): 

1.) PB using the AMBER suite.
2.) PB using APBS and pdb2pqr (http://www.poissonboltzmann.org/apbs)

This script will create a folder in the actual directory with the
folder name as a combination of the used settings.

All energy differences will be written out in the file probes.txt and the 
differences can also be found mapped on the b-factor of the probes in the
file mypdb_bfact.pdb. 

The final prediction can be found in the file mypdb_pred.pdb.

If you have any questions or problems, please contact:
s.schneider@tum.de
"""

parser = OptionParser(description=desc)
parser.add_option("-p", "--pdb", action="store", type="string", dest="pdb", help="The name of the protein pdb file without suffix.")
parser.add_option("--probe", action="store", type="float", dest="probe", default=1.4, help="Size of the probe for desolvation calculation. May differ from the size of the small probe (Default 1.4)")
parser.add_option("--sp", action="store", type="float", dest="sp", default=1.4, help="Small probe size for calculating cavity probes (Default 1.4)")
parser.add_option("--bp", action="store", type="float", dest="bp", default=4.5, help="Big probe size for calculating cavity probes (Default 4.5)")
parser.add_option("-m", action="store", type="int", dest="mode", default=2, help="1 = AMBER GB, "
"2 = AMBER GB using pre-calculated surface points, "
"3 = APBS PB, "
"4 = apbs using pre-calculated surface points, " 
"5 = only create probes, no desolvation calculation, "
"6 = AMBER PB, "
"7 = AMBER PB using pre-calculated surface points, "
"8 = AMBER GB with probes over the entire surface. NOTE: Only profile is created, no prediction. "
"9 = Multiprobe using AMBER GB. NOTE: calculation of desolvation penalties for a patch of probes. Testversion! ")

parser.add_option("--cs", action="store", type="int", dest="cs1", default=30, help="The minimum size of acceptable clusters (Default 30). Increase of the value will result in the exclusion of smaller cavities and smaller values into the acceptance of smaller cavities.")
parser.add_option("--distFac", action="store", type="float", default=1.4, dest="distFac", help="The distance criteria for cluster generation. Using larger distance cut-offs increases the number of accepted "
                  "clusters and allows to neglect small gaps between probes (Default 1.4, fits best if not smaller than the small probe size)")


(options, args) = parser.parse_args()

    
    
"""
########################## SCRIPT ############################
"""

print "####################################################"
print "Now calculating for file " + options.pdb
print "####################################################"


suf = str(options.probe) + "_" + str(options.sp) + "_" + str(options.bp) + "_" + str(options.cs1) + "_" + str(options.distFac)
                        
if options.mode == 1: #CAVITY GB
    try:                
        os.mkdir("gb_cavity_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("gb_cavity_" + suf)        
        os.system("cp ../*.pdb .")               
        gbRunCav = SanderGB_Cavities.AmberGB(options.pdb, options.probe, options.cs1, options.distFac)
        gbRunCav.amberGB(options.sp, options.bp, False, False)    
    except:
        print options.pdb, "failed"
elif options.mode == 2: #CAVITY Reuse GB
    try:                
        os.mkdir("gb_ru_cavity_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("gb_ru_cavity_" + suf)
        os.system("cp ../*.pdb .")   
        os.system("cp ../*.spt .")
        gbRunCav = SanderGB_Cavities.AmberGB(options.pdb, options.probe, options.cs1, options.distFac)
        gbRunCav.amberGB(options.sp, options.bp, True, test)    
    except:
        print options.pdb, "failed"
elif options.mode == 3: #CAVITY APBS
    try:                
        os.mkdir("apbs_cavity_" + suf)
    except:
        pass
    test = False

    try:
        os.chdir("apbs_cavity_" + suf)        
        os.system("cp ../*.pdb .")               
        absRunCav = APBS_Cavities.APBS(options.pdb, options.probe, options.cs1, options.distFac)
        absRunCav.apbsSingleProbeRun(options.sp, options.bp, False)    
    except:
        print options.pdb, "failed"
elif options.mode == 4: #CAVITY REUSE APBS
    try:                
        os.mkdir("apbs_ru_cavity_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("apbs_ru_cavity_" + suf)
        os.system("cp ../*.pdb .") 
        os.system("cp ../*.spt .")
        absRunCavRU = APBS_Cavities.APBS(options.pdb, options.probe, options.cs1, options.distFac)
        absRunCavRU.apbsSingleProbeRun(options.sp, options.bp, True)      
    except:
        print options.pdb, "failed"
elif options.mode == 5: #CAVITY Test GB
    try:                
        os.mkdir("gb_test_cavity_" + suf)
    except:
        pass
    test = True
    reuse = False
    os.chdir("gb_test_cavity_" + suf)
    os.system("cp ../*.pdb .")             
    gbRunCavTest = SanderGB_Cavities.AmberGB(options.pdb, options.probe, options.cs1, options.distFac)
    gbRunCavTest.amberGB(options.sp, options.bp, reuse, test)    
elif options.mode == 6: #CAVITY Sander PB
    try:                
        os.mkdir("pb_cavity_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("pb_cavity_" + suf)                  
        os.system("cp ../*.pdb .")             
    except:
        pass  
    pbRunCav = SanderPB_Cavities.AmberPB(options.pdb, options.probe, options.cs1, options.distFac)
    pbRunCav.amberPB(options.sp, options.bp, False, False)    
elif options.mode == 7: #CAVITY Reuse Sander PB
    try:                
        os.mkdir("pb_ru_cavity_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("pb_ru_cavity_" + suf)
        os.system("cp ../*.pdb .")   
        pbRunCav_ru = SanderPB_Cavities.AmberPB(options.pdb, options.probe, options.cs1, options.distFac)
        pbRunCav_ru.amberPB(options.sp, options.bp, True, test)   
    except:
        print options.pdb, "failed"
elif options.mode == 8: #ALL surface SanderGB                                                                                                                                                               
    try:
        os.mkdir("gb_tot_" + suf)
    except:
        pass
    test = False
    try:
        os.chdir("gb_tot_" + suf)
        os.system("cp ../*.pdb .")                                                                                                                                        
        gbRuntot = SanderGB.AmberGB(options.pdb, options.probe, options.cs1, options.distFac)
        gbRuntot.amberGB(options.sp, options.bp, True, test)
    except:
        print options.pdb, "failed"
