'''
Created on 03.06.2011

@author: sschneider
'''
import math

def distanceNoSQRT(a, b):
    return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z)

def distance(a, b):    
    return math.sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z))

def colorProteinOnBfacWithHydrophilicityKyteAndDoolittle(protein):
    res_standard = {"ALA":1.8, "ARG":-4.5, "ASN":-3.5, "ASP":-3.5, "CYS":2.5, "GLU":-3.5, "GLN":-3.5, "GLY":-0.4, "HIS":-3.2, "ILE":4.5, "LEU":3.8, "LYS":-3.9, "MET":1.9, "PHE":2.8, "PRO":-1.6, "SER":-0.8, "THR":-0.7, "TRP":-0.9, "TYR":-1.3, "VAL":4.2}
    for a in protein.atoms:
        if a.res_name.find("CYX") != -1:
            a.res_name = "CYS"
        if a.res_name.find("HIE") != -1:
            a.res_name = "HIS"
        if a.res_name not in res_standard:
            print "found unknown residue:", a.res_name
            a.temp = -100.0
        a.temp = res_standard[a.res_name]
    return protein


def getProteinBox(protein):
    xh = -1000.0
    yh = -1000.0
    zh = -1000.0
    xl = 1000.0
    yl = 1000.0
    zl = 1000.0
    for a in protein.atoms:
        if a.x > xh:
            xh = a.x
        if a.y > yh:
            yh = a.y
        if a.z > zh:
            zh = a.z
        if a.x < xl:
            xl = a.x
        if a.y < yl:
            yl = a.y
        if a.z < zl:
            zl = a.z
    return (xl, yl, zl), (xh, yh, zh)

def getProteinCenter(protein):
    x = 0.0
    y = 0.0
    z = 0.0
    for a in protein.atoms:
        x += a.x
        y += a.y
        z += a.z
    x /= float(len(protein.atoms))
    y /= float(len(protein.atoms))
    z /= float(len(protein.atoms))
    return (x, y, z)

def getNearest(atom, protein, res_in, res_surf):    
    nearest = None
    near = 10000000.0    
    for a in protein.atoms:
        if a.res_nr in res_in or a.res_nr not in res_surf:
            continue
        d = distance(atom, a)        
        if d < near:
            near = d
            nearest = a.copy()  
    try:      
        return nearest.res_nr
    except:
        return None
    
    
def getSurfaceResiduesFromMetaPPISP(f):
    lines = open(f).readlines()
    surfRes = {}
    for line in lines:
        ele = line.split()
        try:
            if ele[6] == "P" or ele[6] == "N":
                surfRes[int(ele[1])] = float(ele[5])
        except:
            pass
    return surfRes

#def placeProbes(pdb, srad=1.4, nphi=25, ncosth=15):
    

def calcSmallProbes(pdb, srad=1.4):   
    print "init"
    from Structure import Atom     
    nphi = 35;
    ncosth = 25;        
    natoms = len(pdb.atoms)        
    csth = []
    snth = []
    phgh = []
    cos_phigh = []
    sin_phigh = [] 
    neigh = []   
    sasa = []        
    print "start placing probes"
    for ik in range(ncosth):        
        csth.append(-1.0 + ((ik + 1) - 0.5) * (2.0 / (ncosth)))
        snth.append(math.sqrt(1.0 - csth[ik] * csth[ik]))    
    for ik in range(nphi):
        phgh.append(2.0 * math.pi * (ik) / (nphi));    
    for kk in range(nphi):
        cos_phigh.append(math.cos(phgh[kk]))
        sin_phigh.append(math.sin(phgh[kk]))
    
    for i in range(natoms):        
        neigh[:] = []                  
        for j in range(natoms):
            if j == i:
                continue        
            if distance(pdb.atoms[i], pdb.atoms[j]) <= (pdb.atoms[i].vdw + pdb.atoms[j].vdw + 2.0 * srad):
                neigh.append(pdb.atoms[j])    
        for jj in range(ncosth):
            costh = csth[jj];
            sinth = snth[jj];
            for kk in range(nphi):                
                xp = (pdb.atoms[i].vdw + srad) * sinth * cos_phigh[kk];
                yp = (pdb.atoms[i].vdw + srad) * sinth * sin_phigh[kk];
                zp = (pdb.atoms[i].vdw + srad) * costh;
                ll = 0;
                coverd = False;
                while (not coverd and ll < len(neigh)):
                    if (pow((pdb.atoms[i].x + xp - neigh[ll].x), 2) + pow((pdb.atoms[i].y + yp
                            - neigh[ll].y), 2) + pow((pdb.atoms[i].z + zp - neigh[ll].z), 2)) < pow((neigh[ll].vdw + srad), 2):                        
                        coverd = True;                    
                    ll += 1;                
                if (coverd == False):
#                    tmp_p = Point3D.Point3D(pdb.atoms[i].x + xp, pdb.atoms[i].y + yp, pdb.atoms[i].z + zp)
                    tmp_p = Atom.Atom(1, "H", "", "", "", "SPS", "", 1, "", pdb.atoms[i].x + xp, pdb.atoms[i].y + yp, pdb.atoms[i].z + zp, 0.0, 0.0, "", "", "")
                    tmp_bool = False
                    for a in sasa:
                        if distanceNoSQRT(a, tmp_p) < (srad * srad): #* 2.0:# / 2.0:
                            tmp_bool = True
                            break
                    if tmp_bool == False:                                   
                        sasa.append(tmp_p.copy())                                                 
    from Structure import Protein
    from IO import PDBWriter
    PDBWriter.PDBWriter().writePDB("surface.points", Protein.Protein(sasa), False)              
    print "finished placing probes"
    return sasa  


def calcFreeVolume(pdb, srad):   
#    from Structure import Protein
    new_pdb = pdb.copy()   
    for a in new_pdb.atoms:
        a.temp = 0.0  
    nphi = 40;
    ncosth = 30;        
    natoms = len(pdb.atoms)        
    csth = []
    snth = []
    phgh = []
    cos_phigh = []
    sin_phigh = [] 
    neigh = []   
    radii = []
    for a in new_pdb.atoms:
        if a.at_name == "C":
            a.vdw = 1.9
            radii.append(1.9)
        elif a.at_name == "P":
            a.vdw = 2.0
            radii.append(2.0)
        elif a.at_name == "S":
            a.vdw = 2.0
            radii.append(2.0)
        elif a.at_name == "N":
            a.vdw = 1.6
            radii.append(1.6)
        elif a.at_name == "O":
            a.vdw = 1.5
            radii.append(1.5)
        else:
            a.vdw = 0.0
            radii.append(0.0)
            
    sasa = []       
    for i in range(natoms):
        sasa.append(0.0)
    for ik in range(ncosth):        
        csth.append(-1.0 + ((ik + 1) - 0.5) * (2.0 / (ncosth)))
        snth.append(math.sqrt(1.0 - csth[ik] * csth[ik]))    
    for ik in range(nphi):
        phgh.append(2.0 * math.pi * (ik) / (nphi));    
    for kk in range(nphi):
        cos_phigh.append(math.cos(phgh[kk]))
        sin_phigh.append(math.sin(phgh[kk]))
    
    for i in range(natoms):      
                          
        for jj in range(natoms):
            if jj == i:
                continue;
            ccdist = math.sqrt(distanceNoSQRT(new_pdb.atoms[i], new_pdb.atoms[jj]))
            rr = (radii[i] + radii[jj] + 2.0 * srad)            
            if (ccdist <= rr):
                neigh.append(new_pdb.atoms[jj])    
        # print i, len(neigh)
        for jj in range(ncosth):
            costh = csth[jj];
            sinth = snth[jj];
            for kk in range(nphi):  
                phigh = phgh[kk]
                xp = (radii[i] + srad) * sinth * math.cos(phigh)
                yp = (radii[i] + srad) * sinth * math.sin(phigh)
                zp = (radii[i] + srad) * costh              
#                xp = (radii[i] + srad) * sinth * cos_phigh[kk];
#                yp = (radii[i] + srad) * sinth * sin_phigh[kk];
#                zp = (radii[i] + srad) * costh;
                ll = 0
                coverd = False
                while (coverd == False and ll < len(neigh)):                                        
                    if (pow((new_pdb.atoms[i].x + xp - neigh[ll].x), 2) + pow((new_pdb.atoms[i].y + yp
                            - neigh[ll].y), 2) + pow((new_pdb.atoms[i].z + zp - neigh[ll].z), 2)) < pow((neigh[ll].vdw + srad), 2):
                        coverd = True                             
                    ll += 1                
                if coverd == False:
                    sasa[i] += 1.0                                                                   
        neigh[:] = []
    for i in range(natoms):
#        print "surface atom count", new_pdb.atoms[i].at_nr, sasa[i]
        sasa[i] *= pow((radii[i] + srad), 2) * (2.0 / ncosth) * (2.0 * math.pi / nphi)
        new_pdb.atoms[i].temp = sasa[i]

    sasa_new = {}

    for a in new_pdb.atoms:
#        print "atom", a.at_nr, a.temp
        if a.res_nr not in sasa_new:
            sasa_new[a.res_nr] = a.temp
        else:
            sasa_new[a.res_nr] += a.temp                
    return sasa_new 
